@@ -25,7 +25,7 @@ import xml.dom.minidom
 # SET VERSION
 # -----------
 
-version = '3.11.0'
+version = '3.12.1'
 nightbuild = ''
 
 
@@ -71,9 +71,15 @@ main={
     'appHeight': 620,
     'appMaximized': 0,
     'unlockGUI': 0,
+    'layout': 'default',
+    'documentsWidth': 195,
+    'documentsHeight': 195,
+    'peaklistWidth': 195,
+    'peaklistHeight': 195,
     'mzDigits': 4,
     'intDigits': 0,
     'ppmDigits': 1,
+    'dataPrecision': 32,
     'lastDir': '',
     'lastSeqDir': '',
     'errorUnits': 'Da',
@@ -218,8 +224,15 @@ processing={
         'removeIsotopes': 1,
         'removeUnknown': 1,
         'labelEnvelope': 'monoisotopic',
+        'envelopeIntensity': 'maximum',
         'setAsMonoisotopic': 0,
     },
+    'deconvolution':{
+        'massType': 0,
+        'groupWindow': 0.01,
+        'groupPeaks': 1,
+        'forceGroupWindow': 0,
+    }
 }
 
 calibration={
@@ -304,8 +317,10 @@ generator={
     'fwhm': 0.1,
     'points': 10,
     'noise': 0,
+    'forceFwhm': 0,
     'showPeaks': 1,
     'showOverlay': 0,
+    'showFlipped': 0,
 }
 
 mascot={
@@ -612,6 +627,10 @@ def loadConfig(path=os.path.join(confdir, 'config.xml')):
         deisotopingTags = processingTags[0].getElementsByTagName('deisotoping')
         if deisotopingTags:
             _getParams(deisotopingTags[0], processing['deisotoping'])
+        
+        deconvolutionTags = processingTags[0].getElementsByTagName('deconvolution')
+        if deconvolutionTags:
+            _getParams(deconvolutionTags[0], processing['deconvolution'])
     
     # calibration
     calibrationTags = document.getElementsByTagName('calibration')
@@ -746,7 +765,11 @@ def saveConfig(path=os.path.join(confdir, 'config.xml')):
     buff += '    <param name="appWidth" value="%d" type="int" />\n' % (main['appWidth'])
     buff += '    <param name="appHeight" value="%d" type="int" />\n' % (main['appHeight'])
     buff += '    <param name="appMaximized" value="%d" type="int" />\n' % (bool(main['appMaximized']))
-    buff += '    <param name="unlockGUI" value="%d" type="int" />\n' % (bool(main['unlockGUI']))
+    buff += '    <param name="layout" value="%s" type="str" />\n' % (_escape(main['layout']))
+    buff += '    <param name="documentsWidth" value="%d" type="int" />\n' % (main['documentsWidth'])
+    buff += '    <param name="documentsHeight" value="%d" type="int" />\n' % (main['documentsHeight'])
+    buff += '    <param name="peaklistWidth" value="%d" type="int" />\n' % (main['peaklistWidth'])
+    buff += '    <param name="peaklistHeight" value="%d" type="int" />\n' % (main['peaklistHeight'])
     buff += '    <param name="macListCtrlGeneric" value="%d" type="int" />\n' % (bool(main['macListCtrlGeneric']))
     buff += '    <param name="peaklistColumns" value="%s" type="str" />\n' % (';'.join(main['peaklistColumns']))
     buff += '    <param name="mzDigits" value="%d" type="int" />\n' % (main['mzDigits'])
@@ -872,8 +895,15 @@ def saveConfig(path=os.path.join(confdir, 'config.xml')):
     buff += '      <param name="removeIsotopes" value="%d" type="int" />\n' % (bool(processing['deisotoping']['removeIsotopes']))
     buff += '      <param name="removeUnknown" value="%d" type="int" />\n' % (bool(processing['deisotoping']['removeUnknown']))
     buff += '      <param name="labelEnvelope" value="%s" type="str" />\n' % (processing['deisotoping']['labelEnvelope'])
+    buff += '      <param name="envelopeIntensity" value="%s" type="str" />\n' % (processing['deisotoping']['envelopeIntensity'])
     buff += '      <param name="setAsMonoisotopic" value="%d" type="int" />\n' % (bool(processing['deisotoping']['setAsMonoisotopic']))
     buff += '    </deisotoping>\n'
+    buff += '    <deconvolution>\n'
+    buff += '      <param name="massType" value="%d" type="int" />\n' % (processing['deconvolution']['massType'])
+    buff += '      <param name="groupWindow" value="%f" type="float" />\n' % (processing['deconvolution']['groupWindow'])
+    buff += '      <param name="groupPeaks" value="%d" type="int" />\n' % (bool(processing['deconvolution']['groupPeaks']))
+    buff += '      <param name="forceGroupWindow" value="%d" type="int" />\n' % (bool(processing['deconvolution']['forceGroupWindow']))
+    buff += '    </deconvolution>\n'
     buff += '  </processing>\n\n'
     
     # calibration
@@ -958,6 +988,7 @@ def saveConfig(path=os.path.join(confdir, 'config.xml')):
     buff += '    <param name="fwhm" value="%f" type="float" />\n' % (generator['fwhm'])
     buff += '    <param name="points" value="%d" type="int" />\n' % (generator['points'])
     buff += '    <param name="noise" value="%f" type="float" />\n' % (generator['noise'])
+    buff += '    <param name="forceFwhm" value="%d" type="int" />\n' % (bool(generator['forceFwhm']))
     buff += '    <param name="showPeaks" value="%d" type="int" />\n' % (bool(generator['showPeaks']))
     buff += '    <param name="showOverlay" value="%d" type="int" />\n' % (bool(generator['showOverlay']))
     buff += '  </generator>\n\n'
