@@ -361,7 +361,7 @@ class dlgCompoundsEditor(wx.Dialog):
             return
         
         # delete selected group
-        title = 'Do you realy want to delete selected group?'
+        title = 'Do you really want to delete selected group?'
         message = 'All compounds within the group will be lost.'
         buttons = [(wx.ID_CANCEL, "Cancel", 80, False, 15), (wx.ID_OK, "Delete", 80, True, 0)]
         dlg = mwx.dlgMessage(self, title, message, buttons)
@@ -425,7 +425,7 @@ class dlgCompoundsEditor(wx.Dialog):
             return
         
         # delete?
-        title = 'Do you realy want to delete selected compounds?'
+        title = 'Do you really want to delete selected compounds?'
         message = 'Compound definitions will be lost.'
         buttons = [(wx.ID_CANCEL, "Cancel", 80, False, 15), (wx.ID_OK, "Delete", 80, True, 0)]
         dlg = mwx.dlgMessage(self, title, message, buttons)
