@@ -228,7 +228,7 @@ class dlgModificationsEditor(wx.Dialog):
         """Remove selected items."""
         
         # delete?
-        title = 'Do you realy want to delete selected modifications?'
+        title = 'Do you really want to delete selected modifications?'
         message = 'Modification definitions will be lost.'
         buttons = [(wx.ID_CANCEL, "Cancel", 80, False, 15), (wx.ID_OK, "Delete", 80, True, 0)]
         dlg = mwx.dlgMessage(self, title, message, buttons)
