@@ -51,7 +51,6 @@ class document():
         self.institution = ''
         self.instrument = ''
         self.notes = ''
-        self.precision = 4
         
         self.spectrum = mspy.scan()
         self.annotations = []
@@ -199,9 +198,10 @@ class document():
         buff += '  </description>\n\n'
         
         # format spectrum
+        precision = config.main['dataPrecision']
         endian = sys.byteorder
         points = self.spectrum.points
-        mzArray, intArray = self._convertSpectrum(points)
+        mzArray, intArray = self._convertSpectrum(points, precision)
         attributes = 'points="%s"' % len(points)
         if self.spectrum.scanNumber != None:
                 attributes += ' scanNumber="%s"' % self.spectrum.scanNumber
@@ -218,8 +218,8 @@ class document():
         
         buff += '  <spectrum %s>\n' % attributes
         if len(points) > 0:
-            buff += '    <mzArray precision="32" compression="zlib" endian="%s">%s</mzArray>\n' % (endian, mzArray)
-            buff += '    <intArray precision="32" compression="zlib" endian="%s">%s</intArray>\n' % (endian, intArray)
+            buff += '    <mzArray precision="%s" compression="zlib" endian="%s">%s</mzArray>\n' % (precision, endian, mzArray)
+            buff += '    <intArray precision="%s" compression="zlib" endian="%s">%s</intArray>\n' % (precision, endian, intArray)
         buff += '  </spectrum>\n\n'
         
         # format peaklist
@@ -234,6 +234,8 @@ class document():
                 attributes += ' isotope="%d"' % peak.isotope
             if peak.fwhm != None:
                 attributes += ' fwhm="%.6f"' % peak.fwhm
+            if peak.group:
+                attributes += ' group="%s"' % self._escape(peak.group)
             buff += '    <peak %s />\n' % attributes
         buff += '  </peaklist>\n\n'
         
@@ -241,13 +243,13 @@ class document():
         buff += '  <annotations>\n'
         for annot in self.annotations:
             attributes = 'peakMZ="%.6f" peakIntensity="%.6f" peakBaseline="%.6f"' % (annot.mz, annot.ai, annot.base)
-            if annot.charge !=None:
+            if annot.charge != None:
                 attributes += ' charge="%d"' % annot.charge
             if annot.radical:
                 attributes += ' radical="1"'
-            if annot.theoretical !=None:
+            if annot.theoretical != None:
                 attributes += ' calcMZ="%.6f"' % annot.theoretical
-            if annot.formula !=None:
+            if annot.formula != None:
                 attributes += ' formula="%s"' % annot.formula
             buff += '    <annotation %s>%s</annotation>\n' % (attributes, self._escape(annot.label))
         buff += '  </annotations>\n\n'
@@ -276,19 +278,19 @@ class document():
             buff += '      <matches>\n'
             for match in sequence.matches:
                 attributes = 'peakMZ="%.6f" peakIntensity="%.6f" peakBaseline="%.6f"' % (match.mz, match.ai, match.base)
-                if match.charge !=None:
+                if match.charge != None:
                     attributes += ' charge="%d"' % match.charge
                 if match.radical:
                     attributes += ' radical="1"'
-                if match.theoretical !=None:
+                if match.theoretical != None:
                     attributes += ' calcMZ="%.6f"' % match.theoretical
-                if match.formula !=None:
+                if match.formula != None:
                     attributes += ' formula="%s"' % match.formula
-                if match.sequenceRange !=None:
+                if match.sequenceRange != None:
                     attributes += ' sequenceRange="%d-%d"' % tuple(match.sequenceRange)
-                if match.fragmentSerie !=None:
+                if match.fragmentSerie != None:
                     attributes += ' fragmentSerie="%s"' % match.fragmentSerie
-                if match.fragmentIndex !=None:
+                if match.fragmentIndex != None:
                     attributes += ' fragmentIndex="%s"' % match.fragmentIndex
                 buff += '        <match %s>%s</match>\n' % (attributes, self._escape(match.label))
             buff += '      </matches>\n'
@@ -320,7 +322,7 @@ class document():
         points = len(self.spectrum.points)
         peaks = len(self.spectrum.peaklist)
         
-        basePeak = self.spectrum.peaklist.basepeak()
+        basePeak = self.spectrum.peaklist.basepeak
         if basePeak:
             basePeak = basePeak.intensity
         
@@ -494,15 +496,21 @@ class document():
     # ----
     
     
-    def _convertSpectrum(self, spectrum):
+    def _convertSpectrum(self, spectrum, precision='f'):
         """Convert spectrum data to compressed binary format coded by base64."""
         
+        # get precision
+        if precision == 32:
+            precision = 'f'
+        elif precision == 64:
+            precision = 'd'
+        
         # convert data to binary
         mzArray = ''
         intArray = ''
         for point in spectrum:
-            mzArray += struct.pack('f', point[0])
-            intArray += struct.pack('f', point[1])
+            mzArray += struct.pack(precision, point[0])
+            intArray += struct.pack(precision, point[1])
         
         # compress data by gz
         mzArray = zlib.compress(mzArray)
@@ -853,25 +861,41 @@ class parseMSD():
             mzData = None
             mzArrayTags = spectrumTags[0].getElementsByTagName('mzArray')
             if mzArrayTags:
-                compression = mzArrayTags[0].getAttribute('compression')
+                
+                compression = False
+                if mzArrayTags[0].hasAttribute('compression'):
+                    compression = mzArrayTags[0].getAttribute('compression')
+                
+                precision = 'f'
+                if mzArrayTags[0].hasAttribute('precision') and mzArrayTags[0].getAttribute('precision') == '64':
+                    precision = 'd'
+                
                 endian = '<'
                 if mzArrayTags[0].getAttribute('endian') == 'big':
                     endian = '>'
                 
                 mzData = self._getNodeText(mzArrayTags[0])
-                mzData = self._convertDataPoints(mzData, compression, endian)
+                mzData = self._convertDataPoints(mzData, compression, precision, endian)
             
             # get intArray
             intData = None
             intArrayTags = spectrumTags[0].getElementsByTagName('intArray')
             if intArrayTags:
-                compression = intArrayTags[0].getAttribute('compression')
+                
+                compression = False
+                if intArrayTags[0].hasAttribute('compression'):
+                    compression = intArrayTags[0].getAttribute('compression')
+                
+                precision = 'f'
+                if intArrayTags[0].hasAttribute('precision') and intArrayTags[0].getAttribute('precision') == '64':
+                    precision = 'd'
+                
                 endian = '<'
                 if intArrayTags[0].getAttribute('endian') == 'big':
                     endian = '>'
                 
                 intData = self._getNodeText(intArrayTags[0])
-                intData = self._convertDataPoints(intData, compression, endian)
+                intData = self._convertDataPoints(intData, compression, precision, endian)
             
             # check data
             if not mzData or not intData:
@@ -915,6 +939,7 @@ class parseMSD():
                     charge = None
                     isotope = None
                     fwhm = None
+                    group = ''
                     
                     if peakTag.hasAttribute('baseline'):
                         base = float(peakTag.getAttribute('baseline'))
@@ -926,12 +951,14 @@ class parseMSD():
                         isotope = int(peakTag.getAttribute('isotope'))
                     if peakTag.hasAttribute('fwhm'):
                         fwhm = float(peakTag.getAttribute('fwhm'))
+                    if peakTag.hasAttribute('group'):
+                        group = peakTag.getAttribute('group')
                     
                 except ValueError:
                     continue
                 
                 # make peak
-                peak = mspy.peak(mz=mz, ai=ai, base=base, sn=sn, charge=charge, isotope=isotope, fwhm=fwhm)
+                peak = mspy.peak(mz=mz, ai=ai, base=base, sn=sn, charge=charge, isotope=isotope, fwhm=fwhm, group=group)
                 peaklist.append(peak)
         
         # add peaklist to document
@@ -1205,7 +1232,7 @@ class parseMSD():
     
     # UTILITIES
     
-    def _convertDataPoints(self, data, compression, endian):
+    def _convertDataPoints(self, data, compression, precision='f', endian='<'):
         """Convert spectrum data points."""
         
         try:
@@ -1218,8 +1245,8 @@ class parseMSD():
                 data = zlib.decompress(data)
             
             # convert form binary
-            count = len(data) / struct.calcsize(endian + 'f')
-            data = struct.unpack(endian + 'f' * count, data[0:len(data)])
+            count = len(data) / struct.calcsize(endian + precision)
+            data = struct.unpack(endian + precision * count, data[0:len(data)])
             
             return data
         
