@@ -30,6 +30,7 @@ if wx.Platform == '__WXMAC__':
 
 # file
 ID_documentNew = wx.NewId()
+ID_documentNewFromClipboard = wx.NewId()
 ID_documentDuplicate = wx.NewId()
 ID_documentOpen = wx.NewId()
 ID_documentRecent = wx.NewId()
@@ -66,6 +67,7 @@ ID_documentRecent9 = wx.NewId()
 ID_documentRecentClear = wx.NewId()
 
 HK_documentNew = '\tCtrl+N'
+HK_documentNewFromClipboard = '\tShift+Ctrl+N'
 HK_documentOpen = '\tCtrl+O'
 HK_documentClose = '\tCtrl+W'
 HK_documentCloseAll = '\tShift+Ctrl+W'
@@ -111,6 +113,7 @@ ID_viewPeaklistColumnZ = wx.NewId()
 ID_viewPeaklistColumnMass = wx.NewId()
 ID_viewPeaklistColumnFwhm = wx.NewId()
 ID_viewPeaklistColumnResol = wx.NewId()
+ID_viewPeaklistColumnGroup = wx.NewId()
 
 HK_viewPosBars = '\tAlt+Ctrl+P'
 HK_viewGel = '\tAlt+Ctrl+G'
@@ -128,15 +131,17 @@ HK_viewCanvasProperties = '\tCtrl+J'
 ID_processingUndo = wx.NewId()
 ID_processingPeakpicking = wx.NewId()
 ID_processingDeisotoping = wx.NewId()
+ID_processingDeconvolution = wx.NewId()
 ID_processingBaseline = wx.NewId()
 ID_processingSmoothing = wx.NewId()
 ID_processingCrop = wx.NewId()
 ID_processingMath = wx.NewId()
-ID_processingSwap = wx.NewId()
+ID_toolsSwapData = wx.NewId()
 
 HK_processingUndo = '\tCtrl+Z'
 HK_processingPeakpicking = '\tCtrl+F'
 HK_processingDeisotoping = '\tCtrl+D'
+HK_processingDeconvolution = ''
 HK_processingSmoothing = '\tCtrl+G'
 HK_processingBaseline = '\tCtrl+B'
 
@@ -156,8 +161,6 @@ ID_sequenceMatchPattern = wx.NewId()
 ID_sequenceMatchesDelete = wx.NewId()
 ID_sequenceMatchesCalibrateBy = wx.NewId()
 
-HK_sequenceNew = '\tShift+Ctrl+N'
-
 # tools
 ID_toolsProcessing = wx.NewId()
 ID_toolsCalibration = wx.NewId()
@@ -222,6 +225,14 @@ ID_linksProspector = wx.NewId()
 ID_linksUniMod = wx.NewId()
 ID_linksUniProt = wx.NewId()
 
+# window
+ID_windowMaximize = wx.NewId()
+ID_windowMinimize = wx.NewId()
+ID_windowLayout1 = wx.NewId()
+ID_windowLayout2 = wx.NewId()
+ID_windowLayout3 = wx.NewId()
+ID_windowLayout4 = wx.NewId()
+
 # help
 ID_helpAbout = wx.ID_ABOUT
 ID_helpHomepage = wx.NewId()
