@@ -153,29 +153,21 @@ def loadImages():
         lib['toolsSave'] = tools.GetSubBitmap(wx.Rect(22, 0, 22, 22))
         lib['toolsPrint'] = tools.GetSubBitmap(wx.Rect(44, 0, 22, 22))
         
-        lib['toolsMath'] = tools.GetSubBitmap(wx.Rect(0, 22, 22, 22))
-        lib['toolsCrop'] = tools.GetSubBitmap(wx.Rect(22, 22, 22, 22))
-        lib['toolsSmoothing'] = tools.GetSubBitmap(wx.Rect(44, 22, 22, 22))
-        lib['toolsBaseline'] = tools.GetSubBitmap(wx.Rect(66, 22, 22, 22))
-        lib['toolsPeakpicking'] = tools.GetSubBitmap(wx.Rect(88, 22, 22, 22))
-        lib['toolsDeisotoping'] = tools.GetSubBitmap(wx.Rect(110, 22, 22, 22))
-        lib['toolsCalibration'] = tools.GetSubBitmap(wx.Rect(132, 22, 22, 22))
-        lib['toolsSequence'] = tools.GetSubBitmap(wx.Rect(0, 44, 22, 22))
-        lib['toolsDigest'] = tools.GetSubBitmap(wx.Rect(22, 44, 22, 22))
-        lib['toolsFragment'] = tools.GetSubBitmap(wx.Rect(44, 44, 22, 22))
-        lib['toolsSearch'] = tools.GetSubBitmap(wx.Rect(66, 44, 22, 22))
-        lib['toolsMassCalculator'] = tools.GetSubBitmap(wx.Rect(0, 66, 22, 22))
-        lib['toolsCompoundsSearch'] = tools.GetSubBitmap(wx.Rect(22, 66, 22, 22))
-        lib['toolsPeakDifferences'] = tools.GetSubBitmap(wx.Rect(44, 66, 22, 22))
-        lib['toolsComparePeaklists'] = tools.GetSubBitmap(wx.Rect(66, 66, 22, 22))
-        lib['toolsMascot'] = tools.GetSubBitmap(wx.Rect(88, 66, 22, 22))
-        lib['toolsProfound'] = tools.GetSubBitmap(wx.Rect(110, 66, 22, 22))
-        lib['toolsPresets'] = tools.GetSubBitmap(wx.Rect(132, 66, 22, 22))
-        lib['toolsMassFilter'] = tools.GetSubBitmap(wx.Rect(154, 66, 22, 22))
-        lib['toolsSpectrumGenerator'] = tools.GetSubBitmap(wx.Rect(176, 66, 22, 22))
-        lib['toolsDocumentInfo'] = tools.GetSubBitmap(wx.Rect(0, 88, 22, 22))
-        lib['toolsDocumentReport'] = tools.GetSubBitmap(wx.Rect(22, 88, 22, 22))
-        lib['toolsDocumentExport'] = tools.GetSubBitmap(wx.Rect(44, 88, 22, 22))
+        lib['toolsProcessing'] = tools.GetSubBitmap(wx.Rect(66, 0, 22, 22))
+        lib['toolsCalibration'] = tools.GetSubBitmap(wx.Rect(88, 0, 22, 22))
+        lib['toolsSequence'] = tools.GetSubBitmap(wx.Rect(110, 0, 22, 22))
+        lib['toolsMassCalculator'] = tools.GetSubBitmap(wx.Rect(132, 0, 22, 22))
+        lib['toolsCompoundsSearch'] = tools.GetSubBitmap(wx.Rect(154, 0, 22, 22))
+        lib['toolsPeakDifferences'] = tools.GetSubBitmap(wx.Rect(176, 0, 22, 22))
+        lib['toolsComparePeaklists'] = tools.GetSubBitmap(wx.Rect(198, 0, 22, 22))
+        lib['toolsMascot'] = tools.GetSubBitmap(wx.Rect(220, 0, 22, 22))
+        lib['toolsProfound'] = tools.GetSubBitmap(wx.Rect(242, 0, 22, 22))
+        lib['toolsDocumentInfo'] = tools.GetSubBitmap(wx.Rect(264, 0, 22, 22))
+        lib['toolsDocumentReport'] = tools.GetSubBitmap(wx.Rect(286, 0, 22, 22))
+        lib['toolsDocumentExport'] = tools.GetSubBitmap(wx.Rect(308, 0, 22, 22))
+        lib['toolsPresets'] = tools.GetSubBitmap(wx.Rect(330, 0, 22, 22))
+        lib['toolsMassFilter'] = tools.GetSubBitmap(wx.Rect(352, 0, 22, 22))
+        lib['toolsSpectrumGenerator'] = tools.GetSubBitmap(wx.Rect(374, 0, 22, 22))
     
     # bottombars
     bottombarsOn = images_lib.getBottombarsOnBitmap()
@@ -279,6 +271,8 @@ def loadImages():
     lib['processingPeakpickingOff'] = toolbarsOff.GetSubBitmap(wx.Rect(116, 132, 29, 22))
     lib['processingDeisotopingOn'] = toolbarsOn.GetSubBitmap(wx.Rect(145, 132, 29, 22))
     lib['processingDeisotopingOff'] = toolbarsOff.GetSubBitmap(wx.Rect(145, 132, 29, 22))
+    lib['processingDeconvolutionOn'] = toolbarsOn.GetSubBitmap(wx.Rect(174, 132, 29, 22))
+    lib['processingDeconvolutionOff'] = toolbarsOff.GetSubBitmap(wx.Rect(174, 132, 29, 22))
     
     lib['sequenceEditorOn'] = toolbarsOn.GetSubBitmap(wx.Rect(0, 154, 29, 22))
     lib['sequenceEditorOff'] = toolbarsOff.GetSubBitmap(wx.Rect(0, 154, 29, 22))
