@@ -70,13 +70,20 @@ presets = {
             },
             'deisotoping':{
                 'maxCharge': 5,
-                'massTolerance': 0.02,
+                'massTolerance': 0.005,
                 'intTolerance': 0.5,
                 'isotopeShift': 0.0,
                 'removeIsotopes': 1,
                 'removeUnknown': 1,
                 'setAsMonoisotopic': 1,
                 'labelEnvelope': 'monoisotopic',
+                'envelopeIntensity': 'maximum',
+            },
+            'deconvolution':{
+                'massType': 0,
+                'groupWindow': 0.001,
+                'groupPeaks': 1,
+                'forceGroupWindow': 0,
             },
         },
         'MALDI-TOF Peptides':{
@@ -112,6 +119,13 @@ presets = {
                 'removeUnknown': 1,
                 'setAsMonoisotopic': 1,
                 'labelEnvelope': 'monoisotopic',
+                'envelopeIntensity': 'maximum',
+            },
+            'deconvolution':{
+                'massType': 0,
+                'groupWindow': 0.05,
+                'groupPeaks': 1,
+                'forceGroupWindow': 0,
             },
         },
         'MALDI-TOF Proteins 5-20 kDa':{
@@ -147,6 +161,13 @@ presets = {
                 'removeUnknown': 0,
                 'setAsMonoisotopic': 0,
                 'labelEnvelope': 'monoisotopic',
+                'envelopeIntensity': 'maximum',
+            },
+            'deconvolution':{
+                'massType': 1,
+                'groupWindow': 2.5,
+                'groupPeaks': 1,
+                'forceGroupWindow': 0,
             },
         },
         'MALDI-TOF PSD':{
@@ -182,6 +203,13 @@ presets = {
                 'removeUnknown': 0,
                 'setAsMonoisotopic': 1,
                 'labelEnvelope': 'monoisotopic',
+                'envelopeIntensity': 'maximum',
+            },
+            'deconvolution':{
+                'massType': 0,
+                'groupWindow': 0.1,
+                'groupPeaks': 1,
+                'forceGroupWindow': 0,
             },
         },
         'MALDI-ICR Peptides':{
@@ -217,6 +245,13 @@ presets = {
                 'removeUnknown': 1,
                 'setAsMonoisotopic': 1,
                 'labelEnvelope': 'monoisotopic',
+                'envelopeIntensity': 'maximum',
+            },
+            'deconvolution':{
+                'massType': 0,
+                'groupWindow': 0.001,
+                'groupPeaks': 1,
+                'forceGroupWindow': 0,
             },
         },
         'MALDI-ICR Low Mass':{
@@ -252,6 +287,13 @@ presets = {
                 'removeUnknown': 1,
                 'setAsMonoisotopic': 1,
                 'labelEnvelope': 'monoisotopic',
+                'envelopeIntensity': 'maximum',
+            },
+            'deconvolution':{
+                'massType': 0,
+                'groupWindow': 0.001,
+                'groupPeaks': 1,
+                'forceGroupWindow': 0,
             },
         },
     },
@@ -651,6 +693,10 @@ def loadPresets(path=os.path.join(config.confdir, 'presets.xml'), clear=False):
                 deisotopingTags = presetsTag.getElementsByTagName('deisotoping')
                 if deisotopingTags:
                     _getParams(deisotopingTags[0], container['processing'][name]['deisotoping'])
+                
+                deconvolutionTags = presetsTag.getElementsByTagName('deconvolution')
+                if deconvolutionTags:
+                    _getParams(deconvolutionTags[0], container['processing'][name]['deconvolution'])
     
     # get modifications presets
     modificationsTags = document.getElementsByTagName('modifications')
@@ -880,8 +926,15 @@ def savePresets(path=os.path.join(config.confdir, 'presets.xml')):
         buff += '        <param name="removeIsotopes" value="%d" type="int" />\n' % (bool(item['deisotoping']['removeIsotopes']))
         buff += '        <param name="removeUnknown" value="%d" type="int" />\n' % (bool(item['deisotoping']['removeUnknown']))
         buff += '        <param name="labelEnvelope" value="%s" type="str" />\n' % (item['deisotoping']['labelEnvelope'])
+        buff += '        <param name="envelopeIntensity" value="%s" type="str" />\n' % (item['deisotoping']['envelopeIntensity'])
         buff += '        <param name="setAsMonoisotopic" value="%d" type="int" />\n' % (bool(item['deisotoping']['setAsMonoisotopic']))
         buff += '      </deisotoping>\n'
+        buff += '      <deconvolution>\n'
+        buff += '        <param name="massType" value="%d" type="int" />\n' % (item['deconvolution']['massType'])
+        buff += '        <param name="groupWindow" value="%f" type="float" />\n' % (item['deconvolution']['groupWindow'])
+        buff += '        <param name="groupPeaks" value="%d" type="int" />\n' % (bool(item['deconvolution']['groupPeaks']))
+        buff += '        <param name="forceGroupWindow" value="%d" type="int" />\n' % (bool(item['deconvolution']['forceGroupWindow']))
+        buff += '      </deconvolution>\n'
         buff += '    </presets>\n\n'
     buff += '  </processing>\n\n'
     
