@@ -781,22 +781,22 @@ class validator(wx.PyValidator):
             return
         
         # int only
-        elif self.flag == 'int' and chr(key) in '-0123456789':
+        elif self.flag == 'int' and chr(key) in '-0123456789eE':
             evt.Skip()
             return
         
         # positive int only
-        elif self.flag == 'intPos' and chr(key) in '0123456789':
+        elif self.flag == 'intPos' and chr(key) in '0123456789eE':
             evt.Skip()
             return
         
         # floats only
-        elif self.flag == 'float' and (chr(key) in '-0123456789.'):
+        elif self.flag == 'float' and (chr(key) in '-0123456789.eE'):
             evt.Skip()
             return
         
         # positive floats only
-        elif self.flag == 'floatPos' and (chr(key) in '0123456789.'):
+        elif self.flag == 'floatPos' and (chr(key) in '0123456789.eE'):
             evt.Skip()
             return
         
@@ -877,3 +877,21 @@ class dlgMessage(wx.Dialog):
     
 
 
+
+# HELPERS
+# -------
+
+def layout(parent, sizer):
+    """Ensure correct panel layout - hack."""
+    
+    parent.SetMinSize((-1,-1))
+    sizer.Fit(parent)
+    parent.Layout()
+    
+    size = parent.GetSize()
+    parent.SetSize((size[0]+1, size[1]+1))
+    parent.SetSize(size)
+    parent.SetMinSize(size)
+# ----
+
+
