@@ -369,10 +369,7 @@ class panelCalibration(wx.MiniFrame):
             self.mainSizer.Show(2)
         
         # fit layout
-        self.SetMinSize((-1,-1))
-        self.Layout()
-        self.mainSizer.Fit(self)
-        self.SetMinSize(self.GetSize())
+        mwx.layout(self, self.mainSizer)
     # ----
     
     
@@ -926,7 +923,7 @@ class panelCalibration(wx.MiniFrame):
         
         # convert peaklist
         peaklist = []
-        basePeak = self.currentDocument.spectrum.peaklist.basepeak()
+        basePeak = self.currentDocument.spectrum.peaklist.basepeak
         f = abs(maxY - minY) / basePeak.intensity
         for peak in self.currentDocument.spectrum.peaklist:
             intensity = (peak.intensity * f) + minY
