@@ -227,6 +227,9 @@ class panelDocumentExport(wx.MiniFrame):
         self.peaklistColumnResol_check = wx.CheckBox(panel, -1, "Resolution")
         self.peaklistColumnResol_check.SetValue(config.export['peaklistColumns'].count('resol'))
         
+        self.peaklistColumnGroup_check = wx.CheckBox(panel, -1, "Group")
+        self.peaklistColumnGroup_check.SetValue(config.export['peaklistColumns'].count('group'))
+        
         peaklistSelect_label = wx.StaticText(panel, -1, "Export:")
         self.peaklistSelect_combo = wx.ComboBox(panel, -1, choices=['All Peaks', 'Selected Peaks'], size=(200, mwx.COMBO_HEIGHT), style=wx.CB_READONLY)
         self.peaklistSelect_combo.Select(0)
@@ -251,11 +254,12 @@ class panelDocumentExport(wx.MiniFrame):
         grid1.Add(self.peaklistColumnBase_check, (2,0))
         grid1.Add(self.peaklistColumnInt_check, (3,0))
         grid1.Add(self.peaklistColumnRel_check, (4,0))
-        grid1.Add(self.peaklistColumnSn_check, (0,2))
-        grid1.Add(self.peaklistColumnZ_check, (1,2))
-        grid1.Add(self.peaklistColumnMass_check, (2,2))
-        grid1.Add(self.peaklistColumnFwhm_check, (3,2))
-        grid1.Add(self.peaklistColumnResol_check, (4,2))
+        grid1.Add(self.peaklistColumnSn_check, (5,0))
+        grid1.Add(self.peaklistColumnZ_check, (0,2))
+        grid1.Add(self.peaklistColumnMass_check, (1,2))
+        grid1.Add(self.peaklistColumnFwhm_check, (2,2))
+        grid1.Add(self.peaklistColumnResol_check, (3,2))
+        grid1.Add(self.peaklistColumnGroup_check, (4,2))
         
         grid2 = wx.GridBagSizer(mwx.GRIDBAG_VSPACE, mwx.GRIDBAG_HSPACE)
         grid2.Add(peaklistSelect_label, (0,0), flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
@@ -391,8 +395,7 @@ class panelDocumentExport(wx.MiniFrame):
             self.spectrum_butt.SetBitmapLabel(images.lib['documentExportSpectrumOn'])
         
         # fit layout
-        self.Layout()
-        self.mainSizer.Fit(self)
+        mwx.layout(self, self.mainSizer)
     # ----
     
     
@@ -463,6 +466,7 @@ class panelDocumentExport(wx.MiniFrame):
         self.peaklistColumnMass_check.Enable(enable)
         self.peaklistColumnFwhm_check.Enable(enable)
         self.peaklistColumnResol_check.Enable(enable)
+        self.peaklistColumnGroup_check.Enable(enable)
         self.peaklistSeparator_combo.Enable(enable)
     # ----
     
@@ -639,6 +643,8 @@ class panelDocumentExport(wx.MiniFrame):
                 config.export['peaklistColumns'].append('fwhm')
             if self.peaklistColumnResol_check.IsChecked():
                 config.export['peaklistColumns'].append('resol')
+            if self.peaklistColumnGroup_check.IsChecked():
+                config.export['peaklistColumns'].append('group')
             
             choices = {'Comma':',', 'Semicolon':';', 'Tab':'tab'}
             config.export['peaklistSeparator'] = choices[self.peaklistSeparator_combo.GetValue()]
@@ -751,6 +757,8 @@ class panelDocumentExport(wx.MiniFrame):
                     line += str(peak.fwhm) + separator
                 if 'resol' in config.export['peaklistColumns']:
                     line += str(peak.resolution) + separator
+                if 'group' in config.export['peaklistColumns']:
+                    line += str(peak.group) + separator
                 buff += '%s\n' % (line.rstrip())
         
         # export to mgf
