@@ -319,8 +319,7 @@ class panelDocumentInfo(wx.MiniFrame):
             self.notes_butt.SetBitmapLabel(images.lib['documentInfoNotesOn'])
         
         # fit layout
-        self.Layout()
-        self.mainSizer.Fit(self)
+        mwx.layout(self, self.mainSizer)
     # ----
     
     
@@ -440,24 +439,25 @@ class panelDocumentInfo(wx.MiniFrame):
         # set document
         self.currentDocument = document
         
-        # set values
-        if not self.currentDocument:
-            self.title_value.ChangeValue('')
-            self.date_value.ChangeValue('')
-            self.operator_value.ChangeValue('')
-            self.contact_value.ChangeValue('')
-            self.institution_value.ChangeValue('')
-            self.instrument_value.ChangeValue('')
-            self.scanNumber_value.ChangeValue('')
-            self.retentionTime_value.ChangeValue('')
-            self.msLevel_value.ChangeValue('')
-            self.precursorMZ_value.ChangeValue('')
-            self.precursorCharge_value.ChangeValue('')
-            self.polarity_combo.SetValue('Unknown')
-            self.points_value.ChangeValue('')
-            self.peaklist_value.ChangeValue('')
-            self.notes_value.ChangeValue('')
-        else:
+        # clear previous values
+        self.title_value.ChangeValue('')
+        self.date_value.ChangeValue('')
+        self.operator_value.ChangeValue('')
+        self.contact_value.ChangeValue('')
+        self.institution_value.ChangeValue('')
+        self.instrument_value.ChangeValue('')
+        self.scanNumber_value.ChangeValue('')
+        self.retentionTime_value.ChangeValue('')
+        self.msLevel_value.ChangeValue('')
+        self.precursorMZ_value.ChangeValue('')
+        self.precursorCharge_value.ChangeValue('')
+        self.polarity_combo.SetValue('Unknown')
+        self.points_value.ChangeValue('')
+        self.peaklist_value.ChangeValue('')
+        self.notes_value.ChangeValue('')
+        
+        # set document values
+        if self.currentDocument:
             self.title_value.ChangeValue(document.title)
             self.date_value.ChangeValue(document.date)
             self.operator_value.ChangeValue(document.operator)
