@@ -419,10 +419,7 @@ class panelMassCalculator(wx.MiniFrame):
             self.save_butt.Enable(True)
         
         # fit layout
-        self.SetMinSize((-1,-1))
-        self.Layout()
-        self.mainSizer.Fit(self)
-        self.SetMinSize(self.GetSize())
+        mwx.layout(self, self.mainSizer)
     # ----
     
     
@@ -482,8 +479,6 @@ class panelMassCalculator(wx.MiniFrame):
             self.currentPatternScan = None
             self.currentPatternProfile = None
             self.currentPatternPeaks = None
-            self.updateSummary()
-            self.updateIonsList()
             self.updatePatternCanvas()
             return
         
@@ -912,7 +907,7 @@ class panelMassCalculator(wx.MiniFrame):
                 threshold=config.masscalc['patternThreshold'], \
                 charge=charge, \
                 agentFormula=config.masscalc['ionseriesAgent'], \
-                agentCharge=config.masscalc['ionseriesAgentCharge']\
+                agentCharge=config.masscalc['ionseriesAgentCharge'] \
             )
             
             # make peaklist from pattern
@@ -923,6 +918,7 @@ class panelMassCalculator(wx.MiniFrame):
                     mz=isotope[0], \
                     ai=ai, \
                     base=config.masscalc['patternBaseline'], \
+                    fwhm=config.masscalc['patternFwhm'], \
                     charge=charge \
                 )
                 peaklist.append(peak)
@@ -935,15 +931,12 @@ class panelMassCalculator(wx.MiniFrame):
                     mz=isotope.mz, \
                     ai=isotope.ai, \
                     base=isotope.base, \
-                    fwhm=config.masscalc['patternFwhm'] \
+                    fwhm=isotope.fwhm \
                 )
                 self.currentPatternPeaks.append(peak)
             
             # make profile spectrum
-            self.currentPatternProfile = mspy.profile( \
-                peaklist=peaklist, \
-                fwhm=config.masscalc['patternFwhm'] \
-            )
+            self.currentPatternProfile = mspy.profile(peaklist)
             
             # make final spectrum
             self.currentPatternScan = mspy.scan(points=self.currentPatternProfile, peaks=peaklist)
