@@ -347,10 +347,7 @@ class panelMatch(wx.MiniFrame):
             self.summary_butt.SetBitmapLabel(images.lib['matchSummaryOn'])
         
         # fit layout
-        self.SetMinSize((-1,-1))
-        self.Layout()
-        self.mainSizer.Fit(self)
-        self.SetMinSize(self.GetSize())
+        mwx.layout(self, self.mainSizer)
     # ----
     
     
@@ -718,7 +715,7 @@ class panelMatch(wx.MiniFrame):
         
         # convert peaklist
         peaklist = []
-        basePeak = self.currentPeaklist.basepeak()
+        basePeak = self.currentPeaklist.basepeak
         f = abs(maxY - minY) / basePeak.intensity
         for peak in self.currentPeaklist:
             intensity = (peak.intensity * f) + minY
