@@ -206,6 +206,11 @@ class panelPeaklist(wx.Panel):
                 self.peakList.SetColumnWidth(x, 60)
                 x += 1
             
+            elif column == 'group':
+                self.peakList.InsertColumn(x, "group", wx.LIST_FORMAT_LEFT)
+                self.peakList.SetColumnWidth(x, 60)
+                x += 1
+            
             else:
                 continue
     # ----
@@ -248,6 +253,11 @@ class panelPeaklist(wx.Panel):
         peakFwhm_label.SetFont(wx.SMALL_FONT)
         self.peakFwhm_value.SetFont(wx.SMALL_FONT)
         
+        peakGroup_label = wx.StaticText(panel, -1, "group:")
+        self.peakGroup_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), style=wx.TE_PROCESS_ENTER)
+        peakGroup_label.SetFont(wx.SMALL_FONT)
+        self.peakGroup_value.SetFont(wx.SMALL_FONT)
+        
         self.peakMonoisotopic_check = wx.CheckBox(panel, -1, "monoisotopic")
         self.peakMonoisotopic_check.SetValue(True)
         self.peakMonoisotopic_check.SetFont(wx.SMALL_FONT)
@@ -275,7 +285,9 @@ class panelPeaklist(wx.Panel):
         grid.Add(self.peakCharge_value, (4,1), flag=wx.EXPAND)
         grid.Add(peakFwhm_label, (5,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
         grid.Add(self.peakFwhm_value, (5,1), flag=wx.EXPAND)
-        grid.Add(self.peakMonoisotopic_check, (6,1), flag=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(peakGroup_label, (6,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.peakGroup_value, (6,1), flag=wx.EXPAND)
+        grid.Add(self.peakMonoisotopic_check, (7,1), flag=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL)
         grid.AddGrowableCol(1)
         
         buttons = wx.BoxSizer(wx.HORIZONTAL)
@@ -333,6 +345,7 @@ class panelPeaklist(wx.Panel):
         menu.Append(ID_viewPeaklistColumnMass, "Mass", "", wx.ITEM_CHECK)
         menu.Append(ID_viewPeaklistColumnFwhm, "FWHM", "", wx.ITEM_CHECK)
         menu.Append(ID_viewPeaklistColumnResol, "Resolution", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnGroup, "Group", "", wx.ITEM_CHECK)
         
         menu.Check(ID_viewPeaklistColumnMz, bool('mz' in config.main['peaklistColumns']))
         menu.Check(ID_viewPeaklistColumnAi, bool('ai' in config.main['peaklistColumns']))
@@ -344,6 +357,7 @@ class panelPeaklist(wx.Panel):
         menu.Check(ID_viewPeaklistColumnMass, bool('mass' in config.main['peaklistColumns']))
         menu.Check(ID_viewPeaklistColumnFwhm, bool('fwhm' in config.main['peaklistColumns']))
         menu.Check(ID_viewPeaklistColumnResol, bool('resol' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnGroup, bool('group' in config.main['peaklistColumns']))
         
         self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnMz)
         self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnAi)
@@ -355,6 +369,7 @@ class panelPeaklist(wx.Panel):
         self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnMass)
         self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnFwhm)
         self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnResol)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnGroup)
         
         self.PopupMenu(menu)
         menu.Destroy()
@@ -466,7 +481,7 @@ class panelPeaklist(wx.Panel):
             self.mainSizer.Hide(1)
             self.Layout()
         
-        # show peak editind panel
+        # show peak editing panel
         else:
             self.editPeak_butt.SetBitmapLabel(images.lib['peaklistEditorOn'])
             self.mainSizer.Show(1)
@@ -672,6 +687,8 @@ class panelPeaklist(wx.Panel):
                     row.append(peak.fwhm)
                 elif column == 'resol':
                     row.append(peak.resolution)
+                elif column == 'group':
+                    row.append(peak.group)
                 else:
                     continue
             
@@ -744,11 +761,15 @@ class panelPeaklist(wx.Panel):
                 if item[x]:
                     data = '%0.0f' % (item[x])
             
+            elif column == 'group':
+                if item[x] != None:
+                    data = unicode(item[x])
+            
             else:
                 continue
             
             # add data
-            if x== 0 and insert:
+            if x == 0 and insert:
                 self.peakList.InsertStringItem(row, data)
             else:
                 self.peakList.SetStringItem(row, x, data)
@@ -767,6 +788,7 @@ class panelPeaklist(wx.Panel):
         self.peakSN_value.SetValue('')
         self.peakCharge_value.SetValue('')
         self.peakFwhm_value.SetValue('')
+        self.peakGroup_value.SetValue('')
         self.peakMonoisotopic_check.SetValue(True)
         self.peakReplace_butt.Enable(False)
         
@@ -781,6 +803,8 @@ class panelPeaklist(wx.Panel):
                 self.peakCharge_value.SetValue(str(peak.charge))
             if peak.fwhm:
                 self.peakFwhm_value.SetValue(str(round(peak.fwhm,6)))
+            if peak.group:
+                self.peakGroup_value.SetValue(unicode(peak.group))
             if peak.isotope == 0:
                 self.peakMonoisotopic_check.SetValue(True)
             else:
@@ -800,6 +824,7 @@ class panelPeaklist(wx.Panel):
             sn = self.peakSN_value.GetValue()
             charge = self.peakCharge_value.GetValue()
             fwhm = self.peakFwhm_value.GetValue()
+            group = self.peakGroup_value.GetValue()
             monoisotope = self.peakMonoisotopic_check.GetValue()
             
             # format data
@@ -826,13 +851,16 @@ class panelPeaklist(wx.Panel):
             else:
                 fwhm = None
             
+            if not group:
+                group = ''
+            
             if monoisotope:
                 isotope = 0
             else:
                 isotope = None
             
             # make peak
-            peak = mspy.peak(mz=mz, ai=ai, base=base, sn=sn, charge=charge, isotope=isotope, fwhm=fwhm)
+            peak = mspy.peak(mz=mz, ai=ai, base=base, sn=sn, charge=charge, isotope=isotope, fwhm=fwhm, group=group)
             return peak
         
         except:
@@ -894,6 +922,8 @@ class panelPeaklist(wx.Panel):
                     line += str(peak.fwhm) + '\t'
                 if 'resol' in config.export['peaklistColumns']:
                     line += str(peak.resolution) + '\t'
+                if 'group' in config.export['peaklistColumns']:
+                    line += unicode(peak.group) + '\t'
                 buff += '%s\n' % (line.rstrip())
             
             # make text object for data
@@ -1057,6 +1087,9 @@ class dlgCopy(wx.Dialog):
         self.peaklistColumnResol_check = wx.CheckBox(self, -1, "Resolution")
         self.peaklistColumnResol_check.SetValue(config.export['peaklistColumns'].count('resol'))
         
+        self.peaklistColumnGroup_check = wx.CheckBox(self, -1, "Group")
+        self.peaklistColumnGroup_check.SetValue(config.export['peaklistColumns'].count('group'))
+        
         cancel_butt = wx.Button(self, wx.ID_CANCEL, "Cancel")
         copy_butt = wx.Button(self, wx.ID_OK, "Copy")
         copy_butt.SetDefault()
@@ -1068,11 +1101,12 @@ class dlgCopy(wx.Dialog):
         grid.Add(self.peaklistColumnBase_check, (2,0))
         grid.Add(self.peaklistColumnInt_check, (3,0))
         grid.Add(self.peaklistColumnRel_check, (4,0))
-        grid.Add(self.peaklistColumnSn_check, (0,2))
-        grid.Add(self.peaklistColumnZ_check, (1,2))
-        grid.Add(self.peaklistColumnMass_check, (2,2))
-        grid.Add(self.peaklistColumnFwhm_check, (3,2))
-        grid.Add(self.peaklistColumnResol_check, (4,2))
+        grid.Add(self.peaklistColumnSn_check, (5,0))
+        grid.Add(self.peaklistColumnZ_check, (0,2))
+        grid.Add(self.peaklistColumnMass_check, (1,2))
+        grid.Add(self.peaklistColumnFwhm_check, (2,2))
+        grid.Add(self.peaklistColumnResol_check, (3,2))
+        grid.Add(self.peaklistColumnGroup_check, (4,2))
         
         staticSizer.Add(grid, 0, wx.ALL, 5)
         
@@ -1113,6 +1147,8 @@ class dlgCopy(wx.Dialog):
             config.export['peaklistColumns'].append('fwhm')
         if self.peaklistColumnResol_check.IsChecked():
             config.export['peaklistColumns'].append('resol')
+        if self.peaklistColumnGroup_check.IsChecked():
+            config.export['peaklistColumns'].append('group')
     # ----
     
 
