@@ -69,6 +69,7 @@ class panelProcessing(wx.MiniFrame):
         smoothing = self.makeSmoothingPanel()
         peakpicking = self.makePeakpickingPanel()
         deisotoping = self.makeDeisotopingPanel()
+        deconvolution = self.makeDeconvolutionPanel()
         gauge = self.makeGaugePanel()
         
         # pack element
@@ -80,6 +81,7 @@ class panelProcessing(wx.MiniFrame):
         self.mainSizer.Add(smoothing, 1, wx.EXPAND, 0)
         self.mainSizer.Add(peakpicking, 1, wx.EXPAND, 0)
         self.mainSizer.Add(deisotoping, 1, wx.EXPAND, 0)
+        self.mainSizer.Add(deconvolution, 1, wx.EXPAND, 0)
         self.mainSizer.Add(gauge, 0, wx.EXPAND, 0)
         
         # hide panels
@@ -90,6 +92,7 @@ class panelProcessing(wx.MiniFrame):
         self.mainSizer.Hide(5)
         self.mainSizer.Hide(6)
         self.mainSizer.Hide(7)
+        self.mainSizer.Hide(8)
         
         # fit layout
         self.mainSizer.Fit(self)
@@ -128,6 +131,10 @@ class panelProcessing(wx.MiniFrame):
         self.deisotoping_butt.SetToolTip(wx.ToolTip("Deisotoping"))
         self.deisotoping_butt.Bind(wx.EVT_BUTTON, self.onToolSelected)
         
+        self.deconvolution_butt = wx.BitmapButton(panel, ID_processingDeconvolution, images.lib['processingDeconvolutionOff'], size=(mwx.TOOLBAR_TOOLSIZE), style=wx.BORDER_NONE)
+        self.deconvolution_butt.SetToolTip(wx.ToolTip("Deconvolution"))
+        self.deconvolution_butt.Bind(wx.EVT_BUTTON, self.onToolSelected)
+        
         self.presets_butt = wx.BitmapButton(panel, -1, images.lib['toolsPresets'], size=(mwx.TOOLBAR_TOOLSIZE), style=wx.BORDER_NONE)
         self.presets_butt.SetToolTip(wx.ToolTip("Processing presets"))
         self.presets_butt.Bind(wx.EVT_BUTTON, self.onPresets)
@@ -148,6 +155,7 @@ class panelProcessing(wx.MiniFrame):
         self.toolbar.Add(self.smoothing_butt, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, mwx.BUTTON_SIZE_CORRECTION)
         self.toolbar.Add(self.peakpicking_butt, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, mwx.BUTTON_SIZE_CORRECTION)
         self.toolbar.Add(self.deisotoping_butt, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, mwx.BUTTON_SIZE_CORRECTION)
+        self.toolbar.Add(self.deconvolution_butt, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, mwx.BUTTON_SIZE_CORRECTION)
         self.toolbar.AddSpacer(20)
         self.toolbar.Add(self.presets_butt, 0, wx.ALIGN_CENTER_VERTICAL)
         self.toolbar.AddStretchSpacer()
@@ -433,13 +441,23 @@ class panelProcessing(wx.MiniFrame):
         self.deisotopingRemoveUnknown_check.SetValue(bool(config.processing['deisotoping']['removeUnknown']))
         
         deisotopingLabelEnvelopeTool_label = wx.StaticText(panel, -1, "Label envelope tool:")
-        self.deisotopingLabelEnvelopeTool_combo = wx.ComboBox(panel, -1, choices=['Monoisotopic Mass', 'All Isotopes', 'Envelope Centroid'], size=(160, mwx.COMBO_HEIGHT), style=wx.CB_READONLY)
+        self.deisotopingLabelEnvelopeTool_combo = wx.ComboBox(panel, -1, choices=['Monoisotopic Mass', 'Envelope Centroid', 'All Isotopes'], size=(160, mwx.COMBO_HEIGHT), style=wx.CB_READONLY)
         self.deisotopingLabelEnvelopeTool_combo.Select(1)
-        choices=['monoisotopic', 'isotopes', 'centroid']
+        choices=['monoisotopic', 'centroid', 'isotopes']
         if config.processing['deisotoping']['labelEnvelope'] in choices:
             self.deisotopingLabelEnvelopeTool_combo.Select(choices.index(config.processing['deisotoping']['labelEnvelope']))
         self.deisotopingLabelEnvelopeTool_combo.Bind(wx.EVT_COMBOBOX, self.getParams)
         
+        deisotopingEnvelopeIntensity_label = wx.StaticText(panel, -1, "Envelope intensity:")
+        self.deisotopingEnvelopeIntensity_combo = wx.ComboBox(panel, -1, choices=['Envelope Maximum', 'Summed Isotopes', 'Averaged Isotopes'], size=(160, mwx.COMBO_HEIGHT), style=wx.CB_READONLY)
+        self.deisotopingEnvelopeIntensity_combo.Select(0)
+        choices=['maximum', 'sum', 'average']
+        if config.processing['deisotoping']['envelopeIntensity'] in choices:
+            self.deisotopingEnvelopeIntensity_combo.Select(choices.index(config.processing['deisotoping']['envelopeIntensity']))
+        self.deisotopingEnvelopeIntensity_combo.Bind(wx.EVT_COMBOBOX, self.getParams)
+        if config.processing['deisotoping']['labelEnvelope'] == 'isotopes':
+            self.deisotopingEnvelopeIntensity_combo.Disable()
+        
         deisotopingSetAsMonoisotopic_label = wx.StaticText(panel, -1, "Set labels as monoisotopes:")
         self.deisotopingSetAsMonoisotopic_check = wx.CheckBox(panel, -1, "")
         self.deisotopingSetAsMonoisotopic_check.SetValue(config.processing['deisotoping']['setAsMonoisotopic'])
@@ -464,8 +482,55 @@ class panelProcessing(wx.MiniFrame):
         grid.Add(wx.StaticLine(panel), (6,0), (1,3), flag=wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
         grid.Add(deisotopingLabelEnvelopeTool_label, (7,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
         grid.Add(self.deisotopingLabelEnvelopeTool_combo, (7,1), (1,2))
-        grid.Add(deisotopingSetAsMonoisotopic_label, (8,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
-        grid.Add(self.deisotopingSetAsMonoisotopic_check, (8,1))
+        grid.Add(deisotopingEnvelopeIntensity_label, (8,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.deisotopingEnvelopeIntensity_combo, (8,1), (1,2))
+        grid.Add(deisotopingSetAsMonoisotopic_label, (9,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.deisotopingSetAsMonoisotopic_check, (9,1))
+        
+        mainSizer = wx.BoxSizer(wx.VERTICAL)
+        mainSizer.Add(grid, 0, wx.ALIGN_CENTER|wx.ALL, mwx.PANEL_SPACE_MAIN)
+        
+        # fit layout
+        mainSizer.Fit(panel)
+        panel.SetSizer(mainSizer)
+        
+        return panel
+    # ----
+    
+    
+    def makeDeconvolutionPanel(self):
+        """Make controls for deconvolution."""
+        
+        panel = wx.Panel(self, -1)
+        
+        # make elements
+        deconvolutionMassType_label = wx.StaticText(panel, -1, "Mass type:")
+        self.deconvolutionMassType_combo = wx.ComboBox(panel, -1, choices=['Monoisotopic', 'Average'], size=(150, mwx.COMBO_HEIGHT), style=wx.CB_READONLY)
+        self.deconvolutionMassType_combo.Select(config.processing['deconvolution']['massType'])
+        
+        deconvolutionGroupWindow_label = wx.StaticText(panel, -1, "Grouping window:")
+        self.deconvolutionGroupWindow_value = wx.TextCtrl(panel, -1, str(config.processing['deconvolution']['groupWindow']), size=(90, -1), validator=mwx.validator('floatPos'))
+        deconvolutionGroupWindowUnits_label = wx.StaticText(panel, -1, "m/z")
+        
+        deconvolutionGroupPeaks_label = wx.StaticText(panel, -1, "Group peaks:")
+        self.deconvolutionGroupPeaks_check = wx.CheckBox(panel, -1, "")
+        self.deconvolutionGroupPeaks_check.SetValue(config.processing['deconvolution']['groupPeaks'])
+        
+        deconvolutionForceGroupWindow_label = wx.StaticText(panel, -1, "Force grouping window:")
+        self.deconvolutionForceGroupWindow_check = wx.CheckBox(panel, -1, "")
+        self.deconvolutionForceGroupWindow_check.SetValue(config.processing['deconvolution']['forceGroupWindow'])
+        
+        # pack elements
+        grid = wx.GridBagSizer(mwx.GRIDBAG_VSPACE, mwx.GRIDBAG_HSPACE)
+        grid.Add(deconvolutionMassType_label, (0,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.deconvolutionMassType_combo, (0,1), (1,2))
+        grid.Add(deconvolutionGroupWindow_label, (1,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.deconvolutionGroupWindow_value, (1,1))
+        grid.Add(deconvolutionGroupWindowUnits_label, (1,2), flag=wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(deconvolutionGroupPeaks_label, (2,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.deconvolutionGroupPeaks_check, (2,1))
+        grid.Add(deconvolutionForceGroupWindow_label, (3,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.deconvolutionForceGroupWindow_check, (3,1))
         
         mainSizer = wx.BoxSizer(wx.VERTICAL)
         mainSizer.Add(grid, 0, wx.ALIGN_CENTER|wx.ALL, mwx.PANEL_SPACE_MAIN)
@@ -541,6 +606,8 @@ class panelProcessing(wx.MiniFrame):
                 tool = 'peakpicking'
             elif evt.GetId() == ID_processingDeisotoping:
                 tool = 'deisotoping'
+            elif evt.GetId() == ID_processingDeconvolution:
+                tool = 'deconvolution'
         
         # set current tool
         self.currentTool = tool
@@ -552,9 +619,10 @@ class panelProcessing(wx.MiniFrame):
         self.mainSizer.Hide(4)
         self.mainSizer.Hide(5)
         self.mainSizer.Hide(6)
+        self.mainSizer.Hide(7)
         
         # hide presets button
-        self.toolbar.Hide(8)
+        self.toolbar.Hide(9)
         
         # set icons off
         self.math_butt.SetBitmapLabel(images.lib['processingMathOff'])
@@ -563,6 +631,7 @@ class panelProcessing(wx.MiniFrame):
         self.smoothing_butt.SetBitmapLabel(images.lib['processingSmoothingOff'])
         self.peakpicking_butt.SetBitmapLabel(images.lib['processingPeakpickingOff'])
         self.deisotoping_butt.SetBitmapLabel(images.lib['processingDeisotopingOff'])
+        self.deconvolution_butt.SetBitmapLabel(images.lib['processingDeconvolutionOff'])
         
         # clear preview
         self.parent.updateTmpSpectrum(None)
@@ -577,14 +646,14 @@ class panelProcessing(wx.MiniFrame):
             
         elif tool == 'crop':
             self.SetTitle("Crop")
-            self.toolbar.Show(8)
+            self.toolbar.Show(9)
             self.mainSizer.Show(2)
             self.crop_butt.SetBitmapLabel(images.lib['processingCropOn'])
             self.preview_butt.Enable(False)
             
         elif tool == 'baseline':
             self.SetTitle("Baseline Correction")
-            self.toolbar.Show(8)
+            self.toolbar.Show(9)
             self.mainSizer.Show(3)
             self.baseline_butt.SetBitmapLabel(images.lib['processingBaselineOn'])
             self.preview_butt.Enable(True)
@@ -592,14 +661,14 @@ class panelProcessing(wx.MiniFrame):
             
         elif tool == 'smoothing':
             self.SetTitle("Smoothing")
-            self.toolbar.Show(8)
+            self.toolbar.Show(9)
             self.mainSizer.Show(4)
             self.smoothing_butt.SetBitmapLabel(images.lib['processingSmoothingOn'])
             self.preview_butt.Enable(True)
             
         elif tool == 'peakpicking':
             self.SetTitle("Peak Picking")
-            self.toolbar.Show(8)
+            self.toolbar.Show(9)
             self.mainSizer.Show(5)
             self.peakpicking_butt.SetBitmapLabel(images.lib['processingPeakpickingOn'])
             self.preview_butt.Enable(False)
@@ -607,14 +676,20 @@ class panelProcessing(wx.MiniFrame):
             
         elif tool == 'deisotoping':
             self.SetTitle("Deisotoping")
-            self.toolbar.Show(8)
+            self.toolbar.Show(9)
             self.mainSizer.Show(6)
             self.deisotoping_butt.SetBitmapLabel(images.lib['processingDeisotopingOn'])
             self.preview_butt.Enable(False)
+            
+        elif tool == 'deconvolution':
+            self.SetTitle("Deconvolution")
+            self.toolbar.Show(9)
+            self.mainSizer.Show(7)
+            self.deconvolution_butt.SetBitmapLabel(images.lib['processingDeconvolutionOn'])
+            self.preview_butt.Enable(False)
         
         # fit layout
-        self.Layout()
-        self.mainSizer.Fit(self)
+        mwx.layout(self, self.mainSizer)
     # ----
     
     
@@ -625,10 +700,10 @@ class panelProcessing(wx.MiniFrame):
         
         if status:
             self.MakeModal(True)
-            self.mainSizer.Show(7)
+            self.mainSizer.Show(8)
         else:
             self.MakeModal(False)
-            self.mainSizer.Hide(7)
+            self.mainSizer.Hide(8)
             self.processing = None
             mspy.start()
         
@@ -716,12 +791,27 @@ class panelProcessing(wx.MiniFrame):
         self.deisotopingRemoveUnknown_check.SetValue(bool(presets['deisotoping']['removeUnknown']))
         self.deisotopingSetAsMonoisotopic_check.SetValue(bool(presets['deisotoping']['setAsMonoisotopic']))
         
-        choices=['monoisotopic', 'isotopes', 'centroid']
+        choices=['monoisotopic', 'centroid', 'isotopes']
         if presets['deisotoping']['labelEnvelope'] in choices:
             self.deisotopingLabelEnvelopeTool_combo.Select(choices.index(presets['deisotoping']['labelEnvelope']))
         else:
             self.deisotopingLabelEnvelopeTool_combo.Select(1)
         
+        choices=['maximum', 'sum', 'average']
+        if presets['deisotoping']['envelopeIntensity'] in choices:
+            self.deisotopingEnvelopeIntensity_combo.Select(choices.index(presets['deisotoping']['envelopeIntensity']))
+        else:
+            self.deisotopingEnvelopeIntensity_combo.Select(0)
+        
+        # set deconvolution
+        self.deconvolutionMassType_combo.Select(presets['deconvolution']['massType'])
+        self.deconvolutionGroupWindow_value.SetValue(str(presets['deconvolution']['groupWindow']))
+        self.deconvolutionGroupPeaks_check.SetValue(bool(presets['deconvolution']['groupPeaks']))
+        self.deconvolutionForceGroupWindow_check.SetValue(bool(presets['deconvolution']['forceGroupWindow']))
+        
+        # readback all params
+        self.getParams()
+        
         # update guides
         self.onBaselineChanged()
         self.onPeakpickingChanged()
@@ -907,20 +997,21 @@ class panelProcessing(wx.MiniFrame):
             return
         
         # check data
-        if self.currentTool in ('baseline', 'smoothing', 'peakpicking') \
-            and not len(self.currentDocument.spectrum.points):
+        if self.currentTool in ('baseline', 'smoothing', 'peakpicking') and not len(self.currentDocument.spectrum.points):
             wx.Bell()
             return
-        elif self.currentTool == 'deisotoping' \
-            and not len(self.currentDocument.spectrum.peaklist):
+        elif self.currentTool in ('deisotoping', 'deconvolution') and not len(self.currentDocument.spectrum.peaklist):
             wx.Bell()
             return
+        elif self.currentTool in ('deconvolution'):
+            if not self.checkChargedPeaks():
+                return
         
         # get all params
         if not self.getParams():
             return
         
-        # check deisotopping
+        # check deisotopping tolerance
         if self.currentTool in ('peakpicking', 'deisotoping'):
             if not self.checkIsotopeMassTolerance():
                 return
@@ -943,6 +1034,8 @@ class panelProcessing(wx.MiniFrame):
             self.processing = threading.Thread(target=self.runApplyPeakpicking)
         elif self.currentTool == 'deisotoping':
             self.processing = threading.Thread(target=self.runApplyDeisotoping)
+        elif self.currentTool == 'deconvolution':
+            self.processing = threading.Thread(target=self.runApplyDeconvolution)
         else:
             return
         
@@ -952,7 +1045,8 @@ class panelProcessing(wx.MiniFrame):
             self.gauge.pulse()
         
         # update gui
-        self.parent.onDocumentChanged(('spectrum', 'notations'))
+        if self.currentTool != 'deconvolution':
+            self.parent.onDocumentChanged(('spectrum', 'notations'))
         
         # hide processing gauge
         self.onProcessing(False)
@@ -962,6 +1056,8 @@ class panelProcessing(wx.MiniFrame):
         
         # clear preview
         self.previewData = None
+        
+        # update peak picking threshold line
         if self.currentTool == 'peakpicking':
             self.onPeakpickingChanged()
     # ----
@@ -972,7 +1068,6 @@ class panelProcessing(wx.MiniFrame):
         
         self.currentDocument = document
         self.previewData = None
-        self.updateAvailableDocuments()
         self.onToolSelected(tool=self.currentTool)
     # ----
     
@@ -1034,15 +1129,37 @@ class panelProcessing(wx.MiniFrame):
             labelEnvelope = self.deisotopingLabelEnvelopeTool_combo.GetValue()
             if labelEnvelope == 'Monoisotopic Mass':
                 config.processing['deisotoping']['labelEnvelope'] = 'monoisotopic'
-            elif labelEnvelope == 'All Isotopes':
-                config.processing['deisotoping']['labelEnvelope'] = 'isotopes'
             elif labelEnvelope == 'Envelope Centroid':
                 config.processing['deisotoping']['labelEnvelope'] = 'centroid'
+            elif labelEnvelope == 'All Isotopes':
+                config.processing['deisotoping']['labelEnvelope'] = 'isotopes'
+            
+            envelopeIntensity = self.deisotopingEnvelopeIntensity_combo.GetValue()
+            if envelopeIntensity == 'Envelope Maximum':
+                config.processing['deisotoping']['envelopeIntensity'] = 'maximum'
+            elif envelopeIntensity == 'Summed Isotopes':
+                config.processing['deisotoping']['envelopeIntensity'] = 'sum'
+            elif envelopeIntensity == 'Averaged Isotopes':
+                config.processing['deisotoping']['envelopeIntensity'] = 'average'
+            
+            if config.processing['deisotoping']['labelEnvelope'] == 'isotopes':
+                self.deisotopingEnvelopeIntensity_combo.Disable()
+            else:
+                self.deisotopingEnvelopeIntensity_combo.Enable()
             
             if config.processing['deisotoping']['maxCharge'] == 0:
                 wx.Bell()
                 return False
             
+            # deconvolution
+            config.processing['deconvolution']['massType'] = 0
+            if self.deconvolutionMassType_combo.GetValue() == 'Average':
+                config.processing['deconvolution']['massType'] = 1
+            
+            config.processing['deconvolution']['groupWindow'] = float(self.deconvolutionGroupWindow_value.GetValue())
+            config.processing['deconvolution']['groupPeaks'] = bool(self.deconvolutionGroupPeaks_check.GetValue())
+            config.processing['deconvolution']['forceGroupWindow'] = bool(self.deconvolutionForceGroupWindow_check.GetValue())
+            
         # ring error bell if error
         except:
             wx.Bell()
@@ -1354,6 +1471,45 @@ class panelProcessing(wx.MiniFrame):
     # ----
     
     
+    def runApplyDeconvolution(self):
+        """Recalculate peak list to singly-charged and make new document."""
+        
+        # run task
+        try:
+            
+            # copy current document
+            docData = copy.deepcopy(self.currentDocument)
+            docData.title += ' - Deconvoluted'
+            docData.format = 'mSD'
+            docData.path = ''
+            docData.dirty = True
+            docData.backup(None)
+            
+            # remove notations
+            del docData.annotations[:]
+            for sequence in docData.sequences:
+                del sequence.matches[:]
+            
+            # deconvolute peaklist
+            massType = ['mo', 'av']
+            docData.spectrum.deconvolute(massType=massType[config.processing['deconvolution']['massType']])
+            
+            # group peaks
+            if config.processing['deconvolution']['groupPeaks']:
+                docData.spectrum.groupPeaks(
+                    window=config.processing['deconvolution']['groupWindow'], \
+                    forceWindow=config.processing['deconvolution']['forceGroupWindow']
+                )
+            
+            # append new document
+            self.parent.onDocumentNew(document=docData, select=False)
+            
+        # task canceled
+        except mspy.ForceQuit:
+            return
+    # ----
+    
+    
     def checkIsotopeMassTolerance(self):
         """Check isotope mass tolerance for cuurent max charge."""
         
@@ -1377,6 +1533,25 @@ class panelProcessing(wx.MiniFrame):
     # ----
     
     
+    def checkChargedPeaks(self):
+        """Check if at least one peak in current peaklist has charge."""
+        
+        # check charge
+        for peak in self.currentDocument.spectrum.peaklist:
+            if peak.charge:
+                return True
+        
+        # show message
+        wx.Bell()
+        message = "Only the peaks with specified charge can be deconvoluted.\nPlease use deisotoping tool or peak editor to set peak charges\nprior to deconvolution."
+        dlg = mwx.dlgMessage(self, title="There are no charged peaks in your peak list.", message=message)
+        dlg.ShowModal()
+        dlg.Destroy()
+        
+        return False
+    # ----
+    
+    
     def makeThresholdLine(self):
         """Make peakpicking threshold line."""
         
