@@ -42,6 +42,7 @@ class panelSpectrum(wx.Panel):
         self.documents = documents
         self.currentDocument = None
         self.currentTmpSpectrum = None
+        self.currentTmpSpectrumFlip = False
         self.currentNotationMarks = None
         self.currentTool = 'measure'
         
@@ -523,7 +524,7 @@ class panelSpectrum(wx.Panel):
         
         # update tmp spectra
         self.updateNotationMarks(self.currentNotationMarks, refresh=False)
-        self.updateTmpSpectrum(self.currentTmpSpectrum, refresh=False)
+        self.updateTmpSpectrum(self.currentTmpSpectrum, flipped=self.currentTmpSpectrumFlip, refresh=False)
         
         # redraw plot
         if refresh:
@@ -578,10 +579,11 @@ class panelSpectrum(wx.Panel):
     # ----
     
     
-    def updateTmpSpectrum(self, points, refresh=True):
+    def updateTmpSpectrum(self, points, flipped=False, refresh=True):
         """Set new data to tmp spectrum."""
         
         self.currentTmpSpectrum = points
+        self.currentTmpSpectrumFlip = flipped
         
         # check spectrum
         if points == None:
@@ -589,7 +591,6 @@ class panelSpectrum(wx.Panel):
         
         # snap to current spectrum
         normalization = None
-        flipped = False
         xOffset = 0
         yOffset = 0
         if self.currentDocument != None and len(points):
@@ -604,7 +605,10 @@ class panelSpectrum(wx.Panel):
                 yOffset = self.documents[self.currentDocument].offset[1]
             
             # flip points
-            flipped = self.documents[self.currentDocument].flipped
+            if flipped:
+                flipped = not self.documents[self.currentDocument].flipped
+            else:
+                flipped = self.documents[self.currentDocument].flipped
         
         # make tmp spectrum
         obj = mspy.plot.points(points, \
@@ -721,7 +725,7 @@ class panelSpectrum(wx.Panel):
         
         # update tmp spectra
         self.updateNotationMarks(self.currentNotationMarks, refresh=False)
-        self.updateTmpSpectrum(self.currentTmpSpectrum, refresh=False)
+        self.updateTmpSpectrum(self.currentTmpSpectrum, flipped=self.currentTmpSpectrumFlip, refresh=False)
         
         # redraw plot
         if refresh:
@@ -926,34 +930,53 @@ class panelSpectrum(wx.Panel):
         if self.documents[self.currentDocument].spectrum.polarity == -1:
             polarity = -1
         
-        # label monoisotopic peak only
-        if config.processing['deisotoping']['labelEnvelope']=='monoisotopic':
+        # label all isotopes
+        if config.processing['deisotoping']['labelEnvelope'] == 'isotopes':
+            groupname = self.documents[self.currentDocument].spectrum.peaklist.nextGroupName()
+            for x, peak in enumerate(peaks):
+                peak.setIsotope(x)
+                peak.setCharge(charge*polarity)
+                peak.setGroup(groupname)
+                self.documents[self.currentDocument].spectrum.peaklist.append(peak)
+        
+        # label monoisotopic peak
+        elif config.processing['deisotoping']['labelEnvelope'] == 'monoisotopic':
+            
             basepeak = peaks[0]
+            sumIntensity = 0
+            sumBase = 0
             for peak in peaks:
-                if peak.ai > basepeak.ai:
+                sumIntensity += peak.intensity
+                sumBase += peak.base
+                if peak.intensity > basepeak.intensity:
                     basepeak = peak
             
+            base = basepeak.base
+            ai = basepeak.ai
+            sn = basepeak.sn
+            if config.processing['deisotoping']['envelopeIntensity'] == 'sum':
+                base = sumBase / len(peaks)
+                ai = base + sumIntensity
+                sn = (ai - base) * basepeak.sn / (basepeak.ai - basepeak.base)
+            elif config.processing['deisotoping']['envelopeIntensity'] == 'average':
+                base = sumBase / len(peaks)
+                ai = base + sumIntensity / len(isotopes)
+                sn = (ai - base) * basepeak.sn / (basepeak.ai - basepeak.base)
+            
             peak = mspy.peak(mz=peaks[0].mz, \
-                ai=basepeak.ai, \
-                base=basepeak.base, \
-                sn=basepeak.sn, \
+                ai=ai, \
+                base=base, \
+                sn=sn, \
                 charge=charge*polarity, \
                 isotope=0, \
-                fwhm=basepeak.fwhm \
+                fwhm=None \
             )
             
             self.documents[self.currentDocument].spectrum.peaklist.append(peak)
         
-        # label all isotopes
-        elif config.processing['deisotoping']['labelEnvelope']=='isotopes':
-            for x, peak in enumerate(peaks):
-                peak.setIsotope(x)
-                peak.setCharge(charge*polarity)
-                self.documents[self.currentDocument].spectrum.peaklist.append(peak)
-        
         # label envelope centroid
-        elif config.processing['deisotoping']['labelEnvelope']=='centroid':
-            peak = mspy.envelopeCentroid(peaks)
+        elif config.processing['deisotoping']['labelEnvelope'] == 'centroid':
+            peak = mspy.envelopeCentroid(peaks, height=0.5, intensity=config.processing['deisotoping']['envelopeIntensity'])
             if peak:
                 peak.setCharge(charge*polarity)
                 self.documents[self.currentDocument].spectrum.peaklist.append(peak)
