@@ -35,7 +35,7 @@ class panelSpectrumGenerator(wx.MiniFrame):
     """Spectrum generator tool."""
     
     def __init__(self, parent):
-        wx.MiniFrame.__init__(self, parent, -1, 'Spectrum Generator', size=(700, 400), style=wx.DEFAULT_FRAME_STYLE & ~ (wx.RESIZE_BORDER | wx.RESIZE_BOX | wx.MAXIMIZE_BOX))
+        wx.MiniFrame.__init__(self, parent, -1, 'Spectrum Generator', size=(700, 400), style=wx.DEFAULT_FRAME_STYLE & ~ (wx.RESIZE_BOX | wx.MAXIMIZE_BOX))
         
         self.parent = parent
         
@@ -59,6 +59,7 @@ class panelSpectrumGenerator(wx.MiniFrame):
         
         # make toolbar
         toolbar = self.makeToolbar()
+        controlbar = self.makeControlbar()
         
         # make panels
         canvas = self.makeSpectrumCanvas()
@@ -67,11 +68,12 @@ class panelSpectrumGenerator(wx.MiniFrame):
         # pack element
         self.mainSizer = wx.BoxSizer(wx.VERTICAL)
         self.mainSizer.Add(toolbar, 0, wx.EXPAND, 0)
+        self.mainSizer.Add(controlbar, 0, wx.EXPAND, 0)
         self.mainSizer.Add(canvas, 1, wx.EXPAND, 0)
         self.mainSizer.Add(gauge, 0, wx.EXPAND, 0)
         
         # hide gauge
-        self.mainSizer.Hide(2)
+        self.mainSizer.Hide(3)
         
         # fit layout
         self.mainSizer.Fit(self)
@@ -86,17 +88,18 @@ class panelSpectrumGenerator(wx.MiniFrame):
         """Make toolbar."""
         
         # init toolbar
-        panel = mwx.bgrPanel(self, -1, images.lib['bgrToolbarNoBorder'], size=(-1, mwx.TOOLBAR_HEIGHT))
+        panel = mwx.bgrPanel(self, -1, images.lib['bgrToolbar'], size=(-1, mwx.TOOLBAR_HEIGHT))
         
         # make elements
-        self.collapse_butt = wx.BitmapButton(panel, -1, images.lib['arrowsDown'], size=(mwx.TOOLBAR_TOOLSIZE), style=wx.BORDER_NONE)
-        self.collapse_butt.Bind(wx.EVT_BUTTON, self.onCollapse)
-        
-        fwhm_label = wx.StaticText(panel, -1, "Peak FWHM:")
+        fwhm_label = wx.StaticText(panel, -1, "Default FWHM:")
         fwhm_label.SetFont(wx.SMALL_FONT)
         self.fwhm_value = wx.TextCtrl(panel, -1, str(config.generator['fwhm']), size=(60, -1), validator=mwx.validator('floatPos'))
         
-        points_label = wx.StaticText(panel, -1, "Peak points:")
+        self.forceFwhm_check = wx.CheckBox(panel, -1, "Force")
+        self.forceFwhm_check.SetFont(wx.SMALL_FONT)
+        self.forceFwhm_check.SetValue(config.generator['forceFwhm'])
+        
+        points_label = wx.StaticText(panel, -1, "Points per peak:")
         points_label.SetFont(wx.SMALL_FONT)
         self.points_value = wx.TextCtrl(panel, -1, str(config.generator['points']), size=(60, -1), validator=mwx.validator('intPos'))
         
@@ -104,16 +107,6 @@ class panelSpectrumGenerator(wx.MiniFrame):
         noise_label.SetFont(wx.SMALL_FONT)
         self.noise_value = wx.TextCtrl(panel, -1, str(config.generator['noise']), size=(80, -1), validator=mwx.validator('floatPos'))
         
-        self.showPeaks_check = wx.CheckBox(panel, -1, "Show peaks")
-        self.showPeaks_check.SetFont(wx.SMALL_FONT)
-        self.showPeaks_check.SetValue(config.generator['showPeaks'])
-        self.showPeaks_check.Bind(wx.EVT_CHECKBOX, self.onShowPeaks)
-        
-        self.showOverlay_check = wx.CheckBox(panel, -1, "Show overlay")
-        self.showOverlay_check.SetFont(wx.SMALL_FONT)
-        self.showOverlay_check.SetValue(config.generator['showOverlay'])
-        self.showOverlay_check.Bind(wx.EVT_CHECKBOX, self.onShowOverlay)
-        
         self.generate_butt = wx.Button(panel, -1, "Generate", size=(-1, mwx.SMALL_BUTTON_HEIGHT))
         self.generate_butt.SetFont(wx.SMALL_FONT)
         self.generate_butt.Bind(wx.EVT_BUTTON, self.onGenerate)
@@ -125,10 +118,9 @@ class panelSpectrumGenerator(wx.MiniFrame):
         # pack elements
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.AddSpacer(mwx.CONTROLBAR_LSPACE)
-        sizer.Add(self.collapse_butt, 0, wx.ALIGN_CENTER_VERTICAL)
-        sizer.AddSpacer(10)
         sizer.Add(fwhm_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, 5)
-        sizer.Add(self.fwhm_value, 0, wx.ALIGN_CENTER_VERTICAL)
+        sizer.Add(self.fwhm_value, 0, wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, 5)
+        sizer.Add(self.forceFwhm_check, 0, wx.ALIGN_CENTER_VERTICAL)
         sizer.AddSpacer(20)
         sizer.Add(points_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, 5)
         sizer.Add(self.points_value, 0, wx.ALIGN_CENTER_VERTICAL)
@@ -136,11 +128,8 @@ class panelSpectrumGenerator(wx.MiniFrame):
         sizer.Add(noise_label, 0, wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, 5)
         sizer.Add(self.noise_value, 0, wx.ALIGN_CENTER_VERTICAL)
         sizer.AddSpacer(20)
-        sizer.Add(self.showPeaks_check, 0, wx.ALIGN_CENTER_VERTICAL)
-        sizer.AddSpacer(20)
-        sizer.Add(self.showOverlay_check, 0, wx.ALIGN_CENTER_VERTICAL)
         sizer.AddStretchSpacer()
-        sizer.AddSpacer(20)
+        sizer.AddSpacer(40)
         sizer.Add(self.generate_butt, 0, wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, 10)
         sizer.Add(self.apply_butt, 0, wx.ALIGN_CENTER_VERTICAL)
         sizer.AddSpacer(mwx.CONTROLBAR_RSPACE)
@@ -154,6 +143,52 @@ class panelSpectrumGenerator(wx.MiniFrame):
     # ----
     
     
+    def makeControlbar(self):
+        """Make controlbar."""
+        
+        # init toolbar
+        panel = mwx.bgrPanel(self, -1, images.lib['bgrControlbar'], size=(-1, mwx.CONTROLBAR_HEIGHT))
+        
+        # make elements
+        self.collapse_butt = wx.BitmapButton(panel, -1, images.lib['arrowsDown'], size=(mwx.TOOLBAR_TOOLSIZE), style=wx.BORDER_NONE)
+        self.collapse_butt.Bind(wx.EVT_BUTTON, self.onCollapse)
+        
+        self.showPeaks_check = wx.CheckBox(panel, -1, "Show individual peaks")
+        self.showPeaks_check.SetFont(wx.SMALL_FONT)
+        self.showPeaks_check.SetValue(config.generator['showPeaks'])
+        self.showPeaks_check.Bind(wx.EVT_CHECKBOX, self.onShowPeaks)
+        
+        self.showOverlay_check = wx.CheckBox(panel, -1, "Show in main window")
+        self.showOverlay_check.SetFont(wx.SMALL_FONT)
+        self.showOverlay_check.SetValue(config.generator['showOverlay'])
+        self.showOverlay_check.Bind(wx.EVT_CHECKBOX, self.onShowOverlay)
+        
+        self.showFlipped_check = wx.CheckBox(panel, -1, "Show flipped")
+        self.showFlipped_check.SetFont(wx.SMALL_FONT)
+        self.showFlipped_check.SetValue(config.generator['showFlipped'])
+        self.showFlipped_check.Bind(wx.EVT_CHECKBOX, self.onShowFlipped)
+        
+        # pack elements
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
+        sizer.AddSpacer(mwx.CONTROLBAR_LSPACE)
+        sizer.Add(self.collapse_butt, 0, wx.ALIGN_CENTER_VERTICAL)
+        sizer.AddSpacer(10)
+        sizer.Add(self.showPeaks_check, 0, wx.ALIGN_CENTER_VERTICAL)
+        sizer.AddSpacer(20)
+        sizer.Add(self.showOverlay_check, 0, wx.ALIGN_CENTER_VERTICAL)
+        sizer.AddSpacer(20)
+        sizer.Add(self.showFlipped_check, 0, wx.ALIGN_CENTER_VERTICAL)
+        sizer.AddSpacer(mwx.CONTROLBAR_RSPACE)
+        
+        mainSizer = wx.BoxSizer(wx.VERTICAL)
+        mainSizer.Add(sizer, 1, wx.EXPAND)
+        mainSizer.Fit(panel)
+        panel.SetSizer(mainSizer)
+        
+        return panel
+    # ----
+    
+    
     def makeSpectrumCanvas(self):
         """Make plot canvas and set defalt parameters."""
         
@@ -228,10 +263,10 @@ class panelSpectrumGenerator(wx.MiniFrame):
         
         if status:
             self.MakeModal(True)
-            self.mainSizer.Show(2)
+            self.mainSizer.Show(3)
         else:
             self.MakeModal(False)
-            self.mainSizer.Hide(2)
+            self.mainSizer.Hide(3)
             self.processing = None
             mspy.start()
         
@@ -272,6 +307,7 @@ class panelSpectrumGenerator(wx.MiniFrame):
         # get params
         if not self.getParams():
             self.updateSpectrumCanvas()
+            self.parent.updateTmpSpectrum(None)
             return
         
         # show processing gauge
@@ -290,6 +326,9 @@ class panelSpectrumGenerator(wx.MiniFrame):
         # update spectrum canvas
         self.updateSpectrumCanvas(rescale=False)
         
+        # update spectrum overlay
+        self.updateSpectrumOverlay()
+        
         # hide processing gauge
         self.onProcessing(False)
         self.generate_butt.Enable(True)
@@ -305,6 +344,18 @@ class panelSpectrumGenerator(wx.MiniFrame):
             wx.Bell()
             return
         
+        # ask to owerwrite spectrum
+        if self.currentDocument.spectrum.hasPoints():
+            title = 'Do you really want to apply generated spectrum\nto current document?'
+            message = 'Previous spectrum points will be lost.'
+            buttons = [(wx.ID_CANCEL, "Cancel", 80, False, 15), (wx.ID_OK, "Apply", 80, True, 0)]
+            dlg = mwx.dlgMessage(self, title, message, buttons)
+            if dlg.ShowModal() != wx.ID_OK:
+                dlg.Destroy()
+                return
+            else:
+                dlg.Destroy()
+        
         # backup document
         self.currentDocument.backup(('spectrum'))
         
@@ -321,11 +372,11 @@ class panelSpectrumGenerator(wx.MiniFrame):
         """Collapse spectrum panel."""
         
         # Show / hide panel
-        if self.mainSizer.IsShown(1):
-            self.mainSizer.Hide(1)
+        if self.mainSizer.IsShown(2):
+            self.mainSizer.Hide(2)
             self.collapse_butt.SetBitmapLabel(images.lib['arrowsRight'])
         else:
-            self.mainSizer.Show(1)
+            self.mainSizer.Show(2)
             self.collapse_butt.SetBitmapLabel(images.lib['arrowsDown'])
         
         # fit layout
@@ -354,10 +405,18 @@ class panelSpectrumGenerator(wx.MiniFrame):
         config.generator['showOverlay'] = self.showOverlay_check.GetValue()
         
         # update tmp spectrum
-        if config.generator['showOverlay']:
-            self.parent.updateTmpSpectrum(self.currentProfile)
-        else:
-            self.parent.updateTmpSpectrum(None)
+        self.updateSpectrumOverlay()
+    # ----
+    
+    
+    def onShowFlipped(self, evt):
+        """Flip profile overlay in main viewer."""
+        
+        # get value
+        config.generator['showFlipped'] = self.showFlipped_check.GetValue()
+        
+        # update tmp spectrum
+        self.updateSpectrumOverlay()
     # ----
     
     
@@ -371,6 +430,7 @@ class panelSpectrumGenerator(wx.MiniFrame):
         
         # update gui
         self.updateSpectrumCanvas()
+        self.parent.updateTmpSpectrum(None)
     # ----
     
     
@@ -382,6 +442,7 @@ class panelSpectrumGenerator(wx.MiniFrame):
             config.generator['fwhm'] = abs(float(self.fwhm_value.GetValue()))
             config.generator['points'] = abs(int(self.points_value.GetValue()))
             config.generator['noise'] = abs(float(self.noise_value.GetValue()))
+            config.generator['forceFwhm'] = self.forceFwhm_check.GetValue()
             return True
             
         except:
@@ -440,7 +501,6 @@ class panelSpectrumGenerator(wx.MiniFrame):
         # check document
         if not self.currentDocument:
             self.spectrumCanvas.draw(self.spectrumContainer)
-            self.parent.updateTmpSpectrum(None)
             return
         
         # get current peaklist
@@ -488,10 +548,17 @@ class panelSpectrumGenerator(wx.MiniFrame):
         
         # draw spectra
         self.spectrumCanvas.draw(self.spectrumContainer, xAxis=xAxis, yAxis=yAxis)
+    # ----
+    
+    
+    def updateSpectrumOverlay(self):
+        """Show / hide profile overlay in main viewer."""
         
-        # show tmp spectrum overlay
-        if config.generator['showOverlay']:
-            self.parent.updateTmpSpectrum(self.currentProfile)
+        # update tmp spectrum
+        if self.currentProfile == None or not config.generator['showOverlay']:
+            self.parent.updateTmpSpectrum(None)
+        else:
+            self.parent.updateTmpSpectrum(self.currentProfile, flipped=config.generator['showFlipped'])
     # ----
     
     
@@ -504,27 +571,30 @@ class panelSpectrumGenerator(wx.MiniFrame):
         # run task
         try:
             
-            # calculate number of points
-            minX = self.currentDocument.spectrum.peaklist[0].mz - 5*config.generator['fwhm']
-            maxX = self.currentDocument.spectrum.peaklist[-1].mz + 5*config.generator['fwhm']
-            datapoints = 0.4 * config.generator['points'] * (maxX - minX) / config.generator['fwhm']
-            
             # make profile spectrum
             self.currentProfile = mspy.profile( \
                 peaklist=self.currentDocument.spectrum.peaklist, \
                 fwhm=config.generator['fwhm'], \
-                points=int(datapoints), \
-                noise=config.generator['noise'] \
+                points=config.generator['points'], \
+                noise=config.generator['noise'], \
+                forceFwhm=config.generator['forceFwhm'] \
             )
             
             # make peak spectra
             self.currentPeaks = []
             for peak in self.currentDocument.spectrum.peaklist:
+                
+                # get fwhm
+                fwhm = config.generator['fwhm']
+                if peak.fwhm and not config.generator['forceFwhm']:
+                    fwhm = peak.fwhm
+                
+                # calc peak
                 points = mspy.gaussian( \
                     mz=peak.mz, \
                     ai=peak.ai, \
                     base=peak.base, \
-                    fwhm=config.generator['fwhm'] \
+                    fwhm=fwhm \
                 )
                 self.currentPeaks.append(points)
         
