@@ -15,17 +15,14 @@
 #     main directory of the program
 # -------------------------------------------------------------------------
 
-# load configuration
-import config
-
 # register basic objects
+from stopper import *
 from blocks import *
 from objects import *
 
 # register modules
-from common import *
+from utilities import *
 from processing import *
-from peakpicking import *
 from mascot import *
 
 # register parsers
@@ -33,4 +30,5 @@ from parser_xy import parseXY
 from parser_mzxml import parseMZXML
 from parser_mzdata import parseMZDATA
 from parser_mzml import parseMZML
+from parser_mgf import parseMGF
 from parser_fasta import parseFASTA
