@@ -335,29 +335,29 @@ enzymes = {
 }
 
 fragments = {
-    'a': fragment( name='a', terminus='N', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='', cTermFormula='C-1O-1H-1', lossFormula='', nTermFilter=True, cTermFilter=True),
+    'a': fragment( name='a', terminus='N', specifity='', nTermFormula='', cTermFormula='C-1O-1H-1', lossFormula='', nTermFilter=True, cTermFilter=True),
     'a-H2O': fragment( name='a-H2O', terminus='N', specifity='STED', nTermFormula='', cTermFormula='C-1O-1H-1', lossFormula='H2O', nTermFilter=True, cTermFilter=True),
     'a-H3PO4': fragment( name='a-H3PO4', terminus='N', specifity='STY', nTermFormula='', cTermFormula='C-1O-1H-1', lossFormula='H3PO4', nTermFilter=True, cTermFilter=True),
     'a-NH3': fragment( name='a-NH3', terminus='N', specifity='RKQN', nTermFormula='', cTermFormula='C-1O-1H-1', lossFormula='NH3', nTermFilter=True, cTermFilter=True),
-    'b': fragment( name='b', terminus='N', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='', cTermFormula='H-1', lossFormula='', nTermFilter=True, cTermFilter=True),
+    'b': fragment( name='b', terminus='N', specifity='', nTermFormula='', cTermFormula='H-1', lossFormula='', nTermFilter=True, cTermFilter=True),
     'b-H2O': fragment( name='b-H2O', terminus='N', specifity='STED', nTermFormula='', cTermFormula='H-1', lossFormula='H2O', nTermFilter=True, cTermFilter=True),
     'b-H3PO4': fragment( name='b-H3PO4', terminus='N', specifity='STY', nTermFormula='', cTermFormula='H-1', lossFormula='H3PO4', nTermFilter=True, cTermFilter=True),
     'b-NH3': fragment( name='b-NH3', terminus='N', specifity='RKQN', nTermFormula='', cTermFormula='H-1', lossFormula='NH3', nTermFilter=True, cTermFilter=True),
-    'c': fragment( name='c', terminus='N', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='', cTermFormula='NH2', lossFormula='', nTermFilter=True, cTermFilter=True),
-    'c-ladder': fragment( name='c-ladder', terminus='N', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='', cTermFormula='OH', lossFormula='', nTermFilter=False, cTermFilter=True),
-    'im': fragment( name='im', terminus='S', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='H', cTermFormula='C-1O-1H-1', lossFormula='', nTermFilter=True, cTermFilter=False),
-    'int': fragment( name='int', terminus='I', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='H', cTermFormula='H-1', lossFormula='', nTermFilter=False, cTermFilter=False),
-    'int-CO': fragment( name='int-CO', terminus='I', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='H', cTermFormula='C-1O-1H-1', lossFormula='', nTermFilter=False, cTermFilter=False),
+    'c': fragment( name='c', terminus='N', specifity='', nTermFormula='', cTermFormula='NH2', lossFormula='', nTermFilter=True, cTermFilter=True),
+    'c-ladder': fragment( name='c-ladder', terminus='N', specifity='', nTermFormula='', cTermFormula='OH', lossFormula='', nTermFilter=False, cTermFilter=True),
+    'im': fragment( name='im', terminus='S', specifity='', nTermFormula='H', cTermFormula='C-1O-1H-1', lossFormula='', nTermFilter=True, cTermFilter=False),
+    'int': fragment( name='int', terminus='I', specifity='', nTermFormula='H', cTermFormula='H-1', lossFormula='', nTermFilter=False, cTermFilter=False),
+    'int-CO': fragment( name='int-CO', terminus='I', specifity='', nTermFormula='H', cTermFormula='C-1O-1H-1', lossFormula='', nTermFilter=False, cTermFilter=False),
     'int-H2O': fragment( name='int-H2O', terminus='I', specifity='STED', nTermFormula='H', cTermFormula='H-1', lossFormula='H2O', nTermFilter=False, cTermFilter=False),
     'int-H3PO4': fragment( name='int-H3PO4', terminus='C', specifity='STY', nTermFormula='H', cTermFormula='H-1', lossFormula='H3PO4', nTermFilter=False, cTermFilter=False),
     'int-NH3': fragment( name='int-NH3', terminus='I', specifity='RKQN', nTermFormula='H', cTermFormula='H-1', lossFormula='NH3', nTermFilter=False, cTermFilter=False),
-    'n-ladder': fragment( name='n-ladder', terminus='C', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='H', cTermFormula='', lossFormula='', nTermFilter=True, cTermFilter=False),
-    'x': fragment( name='x', terminus='C', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='COH-1', cTermFormula='', lossFormula='', nTermFilter=False, cTermFilter=True),
-    'y': fragment( name='y', terminus='C', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='H', cTermFormula='', lossFormula='', nTermFilter=True, cTermFilter=False),
+    'n-ladder': fragment( name='n-ladder', terminus='C', specifity='', nTermFormula='H', cTermFormula='', lossFormula='', nTermFilter=True, cTermFilter=False),
+    'x': fragment( name='x', terminus='C', specifity='', nTermFormula='COH-1', cTermFormula='', lossFormula='', nTermFilter=False, cTermFilter=True),
+    'y': fragment( name='y', terminus='C', specifity='', nTermFormula='H', cTermFormula='', lossFormula='', nTermFilter=True, cTermFilter=False),
     'y-H2O': fragment( name='y-H2O', terminus='C', specifity='STED', nTermFormula='H', cTermFormula='', lossFormula='H2O', nTermFilter=True, cTermFilter=False),
     'y-H3PO4': fragment( name='y-H3PO4', terminus='C', specifity='STY', nTermFormula='H', cTermFormula='', lossFormula='H3PO4', nTermFilter=True, cTermFilter=False),
     'y-NH3': fragment( name='y-NH3', terminus='C', specifity='RKQN', nTermFormula='H', cTermFormula='', lossFormula='NH3', nTermFilter=True, cTermFilter=False),
-    'z': fragment( name='z', terminus='C', specifity='ACDEFGHIKLMNPQRSTVYW', nTermFormula='N-1H-1', cTermFormula='', lossFormula='', nTermFilter=True, cTermFilter=False),
+    'z': fragment( name='z', terminus='C', specifity='', nTermFormula='N-1H-1', cTermFormula='', lossFormula='', nTermFilter=True, cTermFilter=False),
 }
 
 modifications = {
