@@ -204,7 +204,7 @@ class compound:
     # ----
     
     
-    def pattern(self, fwhm=0.1, threshold=0.01, charge=0, agentFormula='H', agentCharge=1, rounding=7):
+    def pattern(self, fwhm=0.1, threshold=0.01, charge=0, agentFormula='H', agentCharge=1):
         """Get isotopic pattern."""
         
         return processing.pattern(self, \
@@ -212,8 +212,8 @@ class compound:
             threshold=threshold, \
             charge=charge, \
             agentFormula=agentFormula, \
-            agentCharge=agentCharge, \
-            rounding=rounding)
+            agentCharge=agentCharge \
+        )
     # ----
     
     
@@ -671,7 +671,7 @@ class sequence:
     # ----
     
     
-    def pattern(self, fwhm=0.1, threshold=0.01, charge=0, agentFormula='H', agentCharge=1, rounding=7):
+    def pattern(self, fwhm=0.1, threshold=0.01, charge=0, agentFormula='H', agentCharge=1):
         """Get isotopic pattern."""
         
         return processing.pattern(self, \
@@ -679,8 +679,8 @@ class sequence:
             threshold=threshold, \
             charge=charge, \
             agentFormula=agentFormula, \
-            agentCharge=agentCharge, \
-            rounding=rounding)
+            agentCharge=agentCharge \
+        )
     # ----
     
     
@@ -1116,7 +1116,7 @@ class sequence:
 class peak:
     """Peak object definition"""
     
-    def __init__(self, mz, ai=0., base=0., sn=None, charge=None, isotope=None, fwhm=None):
+    def __init__(self, mz, ai=0., base=0., sn=None, charge=None, isotope=None, fwhm=None, group=''):
         
         self.mz = float(mz)
         self.ai = float(ai)
@@ -1125,17 +1125,24 @@ class peak:
         self.charge = charge
         self.isotope = isotope
         self.fwhm = fwhm
-        self.relIntensity = 1.
+        self.group = group
+        
         self.childScanNumber = None
         
-        # buffers
+        # set intensity
+        self.intensity = self.ai - self.base
+        self.relIntensity = 1.
+        
+        # set resolution
+        self.resolution = None
+        if self.fwhm:
+            self.resolution = self.mz/self.fwhm
+        
+        # set buffers
         self._mass = None # neutral mass
         
         # user defined params
         self.userParams = {}
-        
-        # clear buffers and set intensity and resolutio
-        self.reset()
     # ----
     
     
@@ -1145,10 +1152,10 @@ class peak:
         # clear mass buffer
         self._mass = None
         
-        # set intensity
+        # update intensity
         self.intensity = self.ai - self.base
         
-        # set resolution
+        # update resolution
         self.resolution = None
         if self.fwhm:
             self.resolution = self.mz/self.fwhm
@@ -1179,22 +1186,39 @@ class peak:
     
     def setMz(self, mz):
         """Set new m/z value."""
+        
+        # update value
         self.mz = mz
-        self.reset()
+        
+        # update resolution
+        self.resolution = None
+        if self.fwhm:
+            self.resolution = self.mz/self.fwhm
+        
+        # clear mass buffer
+        self._mass = None
     # ----
     
     
     def setAi(self, ai):
         """Set new a.i. value."""
+        
+        # update value
         self.ai = ai
-        self.reset()
+        
+        # update intensity
+        self.intensity = self.ai - self.base
     # ----
     
     
     def setBase(self, base):
         """Set new baseline value."""
+        
+        # update value
         self.base = base
-        self.reset()
+        
+        # update intensity
+        self.intensity = self.ai - self.base
     # ----
     
     
@@ -1206,8 +1230,12 @@ class peak:
     
     def setCharge(self, charge):
         """Set new charge value."""
+        
+        # update value
         self.charge = charge
-        self.reset()
+        
+        # clear mass buffer
+        self._mass = None
     # ----
     
     
@@ -1219,8 +1247,20 @@ class peak:
     
     def setFwhm(self, fwhm):
         """Set new fwhm value."""
+        
+        # update value
         self.fwhm = fwhm
-        self.reset()
+        
+        # update resolution
+        self.resolution = None
+        if self.fwhm:
+            self.resolution = self.mz/self.fwhm
+    # ----
+    
+    
+    def setGroup(self, group):
+        """Set new group name value."""
+        self.group = group
     # ----
     
     
@@ -1232,18 +1272,16 @@ class peaklist:
     def __init__(self, peaks=[]):
         
         # check data
-        self._peaks = []
+        self.peaks = []
         for item in peaks:
             item = self._checkPeak(item)
-            self._peaks.append(item)
-        
-        # buffers
-        self._basepeak = None
+            self.peaks.append(item)
         
         # sort peaklist by m/z
-        self._sort()
+        self.sort()
         
         # set basepeak
+        self.basepeak = None
         self._setBasepeak()
         
         # set relative intensities
@@ -1252,7 +1290,7 @@ class peaklist:
     
     
     def __len__(self):
-        return len(self._peaks)
+        return len(self.peaks)
     # ----
     
     
@@ -1262,49 +1300,49 @@ class peaklist:
         item = self._checkPeak(item)
         
         # basepeak is edited - set new
-        if self._peaks[i] is self._basepeak:
-            self._peaks[i] = item
+        if self.peaks[i] is self.basepeak:
+            self.peaks[i] = item
             self._setBasepeak()
             self._setRelativeIntensities()
         
         # new basepeak set
-        elif self._basepeak and item.intensity > self._basepeak.intensity:
-            self._peaks[i] = item
-            self._basepeak = item
+        elif self.basepeak and item.intensity > self.basepeak.intensity:
+            self.peaks[i] = item
+            self.basepeak = item
             self._setRelativeIntensities()
         
         # lower than basepeak
-        elif self._basepeak:
-            item.relIntensity = item.intensity / self._basepeak.intensity
-            self._peaks[i] = item
+        elif self.basepeak:
+            item.relIntensity = item.intensity / self.basepeak.intensity
+            self.peaks[i] = item
         
         # no basepeak set
         else:
-            self._peaks[i] = item
+            self.peaks[i] = item
             self._setBasepeak()
             self._setRelativeIntensities()
         
         # sort peaklist
-        self._sort()
+        self.sort()
     # ----
     
     
     def __getitem__(self, i):
-        return self._peaks[i]
+        return self.peaks[i]
     # ----
     
     
     def __delitem__(self, i):
         
         # delete basepeak
-        if self._peaks[i] is self._basepeak:
-            del self._peaks[i]
+        if self.peaks[i] is self.basepeak:
+            del self.peaks[i]
             self._setBasepeak()
             self._setRelativeIntensities()
         
         # delete others
         else:
-            del self._peaks[i]
+            del self.peaks[i]
     # ----
     
     
@@ -1333,9 +1371,9 @@ class peaklist:
     
     
     def next(self):
-        if self._index < len(self._peaks):
+        if self._index < len(self.peaks):
             self._index += 1
-            return self._peaks[self._index-1]
+            return self.peaks[self._index-1]
         else:
             raise StopIteration
     # ----
@@ -1350,20 +1388,20 @@ class peaklist:
         item = self._checkPeak(item)
         
         # add peak and sort peaklist
-        if self._peaks and self._peaks[-1].mz > item.mz:
-            self._peaks.append(item)
-            self._sort()
+        if self.peaks and self.peaks[-1].mz > item.mz:
+            self.peaks.append(item)
+            self.sort()
         else:
-            self._peaks.append(item)
+            self.peaks.append(item)
         
         # new basepeak set
-        if self._basepeak and item.intensity > self._basepeak.intensity:
-            self._basepeak = item
+        if self.basepeak and item.intensity > self.basepeak.intensity:
+            self.basepeak = item
             self._setRelativeIntensities()
         
         # lower than basepeak
-        elif self._basepeak and self._basepeak.intensity != 0:
-            item.relIntensity = item.intensity / self._basepeak.intensity
+        elif self.basepeak and self.basepeak.intensity != 0:
+            item.relIntensity = item.intensity / self.basepeak.intensity
         
         # no basepeak set
         else:
@@ -1372,6 +1410,15 @@ class peaklist:
     # ----
     
     
+    def reset(self):
+        """Sort peaklist and recalculate basepeak and relative intensities."""
+        
+        self.sort()
+        self._setBasepeak()
+        self._setRelativeIntensities()
+    # ----
+    
+    
     # GETTERS
     
     def duplicate(self):
@@ -1380,20 +1427,38 @@ class peaklist:
     # ----
     
     
-    def basepeak(self):
-        """Return basepeak."""
-        return self._basepeak
+    def nextGroupName(self):
+        """Get available group name."""
+        
+        # get used names
+        used = []
+        for peak in self.peaks:
+            if peak.group != None and not peak.group in used:
+                used.append(peak.group)
+        
+        # generate new name
+        size = 1
+        while True:
+            for name in self._generateGroupNames(size):
+                if not name in used:
+                    return name
+            size += 1
     # ----
     
     
     # MODIFIERS
     
-    def refresh(self):
-        """Sort peaklist and recalculate basepeak and relative intensities."""
+    def sort(self):
+        """Sort peaks according to m/z."""
         
-        self._sort()
-        self._setBasepeak()
-        self._setRelativeIntensities()
+        buff = []
+        for item in self.peaks:
+            buff.append((item.mz, item))
+        buff.sort()
+        
+        self.peaks = []
+        for item in buff:
+            self.peaks.append(item[1])
     # ----
     
     
@@ -1403,7 +1468,7 @@ class peaklist:
         """
         
         # check peaklist
-        if not self._peaks:
+        if not self.peaks:
             return
         
         # delete peaks
@@ -1411,9 +1476,9 @@ class peaklist:
         indexes.reverse()
         relint = False
         for i in indexes:
-            if self._peaks[i] is self._basepeak:
+            if self.peaks[i] is self.basepeak:
                 relint = True
-            del self._peaks[i]
+            del self.peaks[i]
         
         # recalculate basepeak and relative intensities
         if relint:
@@ -1425,8 +1490,8 @@ class peaklist:
     def empty(self):
         """Remove all peaks."""
         
-        del self._peaks[:]
-        self._basepeak = None
+        del self.peaks[:]
+        self.basepeak = None
     # ----
     
     
@@ -1437,12 +1502,12 @@ class peaklist:
         """
         
         # check peaklist
-        if not self._peaks:
+        if not self.peaks:
             return
         
         # get indexes to delete
         indexes = []
-        for x, peak in enumerate(self._peaks):
+        for x, peak in enumerate(self.peaks):
             if peak.mz < minX or peak.mz > maxX:
                 indexes.append(x)
         
@@ -1456,24 +1521,37 @@ class peaklist:
             x: (int or float) multiplier factor
         """
         
-        for peak in self._peaks:
-            peak.ai *= x
-            peak.base *= x
+        # check peaklist
+        if not self.peaks:
+            return
+        
+        # multiply all peaks
+        for peak in self.peaks:
+            peak.setAi(peak.ai * x)
+            peak.setBase(peak.base * x)
+        
+        # update peaklist
+        self._setBasepeak()
+        self._setRelativeIntensities()
     # ----
     
     
     def concatenate(self, peaksB):
         """Add data from given peaklist.
-            peaksB: (peaklist or list or tuple of peaks) peaklist to be added
+            peaksB: (peaklist or list of peaks) peaklist to be added
         """
         
-        # add new peaks
+        # check peaks
+        buff = []
         for peak in copy.deepcopy(peaksB):
             peak = self._checkPeak(peak)
-            self._peaks.appen(peak)
+            buff.append(peak)
         
-        # update final peaklist
-        self._sort()
+        # store peaks
+        self.peaks += buff
+        
+        # update peaklist
+        self.sort()
         self._setBasepeak()
         self._setRelativeIntensities()
     # ----
@@ -1486,11 +1564,11 @@ class peaklist:
         """
         
         # check peaklist
-        if not self._peaks:
+        if not self.peaks:
             return
         
         # apply calibration
-        for peak in self._peaks:
+        for peak in self.peaks:
             peak.setMz(fce(params, peak.mz))
     # ----
     
@@ -1503,13 +1581,99 @@ class peaklist:
             isotopeShift: (float) isotope distance correction (neutral mass)
         """
         
+        # check peaklist
+        if not self.peaks:
+            return
+        
+        # find isotops
         processing.findIsotopes(
             peaklist=self, \
             maxCharge=maxCharge, \
             mzTolerance=mzTolerance, \
             intTolerance=intTolerance, \
             isotopeShift=isotopeShift \
-            )
+        )
+    # ----
+    
+    
+    def deconvolute(self, massType='mo'):
+        """Recalculate peaklist to singly charged.
+            massType: ('mo' or 'av') mass type used for m/z re-calculation
+        """
+        
+        # check peaklist
+        if not self.peaks:
+            return
+        
+        # deconvolute peaklist
+        peaks = processing.deconvolute(
+            peaklist=self, \
+            massType=massType \
+        )
+        
+        # store data
+        self.peaks[:] = peaks.peaks[:]
+        
+        # update peaklist
+        self.sort()
+        self._setBasepeak()
+        self._setRelativeIntensities()
+    # ----
+    
+    
+    def groupPeaks(self, window, forceWindow=False):
+        """Group peaks within specified window.
+            window: (float) default grouping window
+            forceWindow: (bool) use default window for all peaks instead of fwhm
+        """
+        
+        # check peaklist
+        if not self.peaks:
+            return
+        
+        # group peaks
+        buff = []
+        buff.append(self.peaks[0])
+        for current in self.peaks[1:]:
+            previous = buff[-1]
+            
+            # set window
+            win = window
+            if not forceWindow and previous.fwhm and current.fwhm:
+                win = (previous.fwhm + previous.fwhm) / 4.
+            
+            # group with previous peak
+            if (previous.mz + win) > current.mz:
+                
+                # get m/z and intensity
+                intensity = previous.intensity + current.intensity
+                ai = intensity + previous.base
+                mz = (previous.mz*previous.intensity + current.mz*current.intensity) / intensity
+                
+                # set fwhm
+                if previous.fwhm and current.fwhm:
+                    fwhm = (previous.fwhm*previous.intensity + current.fwhm*current.intensity) / intensity
+                    buff[-1].setFwhm(fwhm)
+                
+                # set peak
+                buff[-1].setAi(ai)
+                buff[-1].setMz(mz)
+            
+            # add new peak
+            else:
+                buff.append(current)
+        
+        # remove group names
+        for peak in buff:
+            peak.setGroup('')
+        
+        # store data
+        self.peaks[:] = buff[:]
+        
+        # update peaklist
+        self.sort()
+        self._setBasepeak()
+        self._setRelativeIntensities()
     # ----
     
     
@@ -1521,16 +1685,16 @@ class peaklist:
         """
         
         # check peaklist
-        if not self._peaks:
+        if not self.peaks:
             return
         
         # get absolute threshold
-        threshold = self._basepeak.intensity * relThreshold
+        threshold = self.basepeak.intensity * relThreshold
         threshold = max(threshold, absThreshold)
         
         # get indexes to delete
         indexes = []
-        for x, peak in enumerate(self._peaks):
+        for x, peak in enumerate(self.peaks):
             if peak.intensity < threshold or (peak.sn != None and peak.sn < snThreshold):
                 indexes.append(x)
         
@@ -1547,12 +1711,12 @@ class peaklist:
         """
         
         # check peaklist
-        if not self._peaks:
+        if not self.peaks:
             return
         
         # get possible parent peaks
         candidates = []
-        for peak in self._peaks:
+        for peak in self.peaks:
             if not peak.sn or peak.sn*relThreshold > 3:
                 candidates.append(peak)
             
@@ -1574,7 +1738,7 @@ class peaklist:
             intThreshold = parent.intensity * relThreshold
             
             # get indexes to delete
-            for x, peak in enumerate(self._peaks):
+            for x, peak in enumerate(self.peaks):
                 if (lowMZ < peak.mz < highMZ) and (peak.intensity < intThreshold) and (not x in indexes):
                     indexes.append(x)
                 if peak.mz > highMZ:
@@ -1589,12 +1753,12 @@ class peaklist:
         """Remove isotopes."""
         
         # check peaklist
-        if not self._peaks:
+        if not self.peaks:
             return
         
         # get indexes to delete
         indexes = []
-        for x, peak in enumerate(self._peaks):
+        for x, peak in enumerate(self.peaks):
             if peak.isotope != 0 and peak.charge != None:
                 indexes.append(x)
         
@@ -1607,12 +1771,12 @@ class peaklist:
         """Remove unknown peaks - no charge set."""
         
         # check peaklist
-        if not self._peaks:
+        if not self.peaks:
             return
         
         # get indexes to delete
         indexes = []
-        for x, peak in enumerate(self._peaks):
+        for x, peak in enumerate(self.peaks):
             if peak.charge == None:
                 indexes.append(x)
         
@@ -1623,20 +1787,6 @@ class peaklist:
     
     # HELPERS
     
-    def _sort(self):
-        """Sort data according to mass."""
-        
-        buff = []
-        for item in self._peaks:
-            buff.append((item.mz, item))
-        buff.sort()
-        
-        self._peaks = []
-        for item in buff:
-            self._peaks.append(item[1])
-    # ----
-    
-    
     def _checkPeak(self, item):
         """Check item to be a valid peak."""
         
@@ -1657,16 +1807,16 @@ class peaklist:
         """Get most intens peak."""
         
         # check peaklist
-        if not self._peaks:
-            self._basepeak = None
+        if not self.peaks:
+            self.basepeak = None
             return
         
         # set new basepeak
-        self._basepeak = self._peaks[0]
-        maxInt = self._basepeak.intensity
-        for item in self._peaks[1:]:
+        self.basepeak = self.peaks[0]
+        maxInt = self.basepeak.intensity
+        for item in self.peaks[1:]:
             if item.intensity > maxInt:
-                self._basepeak = item
+                self.basepeak = item
                 maxInt = item.intensity
     # ----
     
@@ -1675,19 +1825,31 @@ class peaklist:
         """Set relative intensities for all peaks."""
         
         # check peaklist
-        if not self._peaks:
+        if not self.peaks:
             return
         
         # set relative intensities
-        maxInt = self._basepeak.intensity
-        for item in self._peaks:
-            if maxInt:
+        maxInt = self.basepeak.intensity
+        if maxInt:
+            for item in self.peaks:
                 item.relIntensity = item.intensity / maxInt
-            else:
+        else:
+            for item in self.peaks:
                 item.relIntensity = 1.
     # ----
     
     
+    def _generateGroupNames(self, size):
+        """Generates serie of group names like A B.. AA AB... AAA AAB.."""
+        
+        pools = ['ABCDEFGHIJKLMNOPQRSTUVWXYZ'] * size
+        result = [[]]
+        for pool in pools:
+            result = [x+[y] for x in result for y in pool]
+        for prod in result:
+            yield ''.join(prod)
+    # ----
+    
 
 
 class scan:
@@ -1969,7 +2131,7 @@ class scan:
             for peak in self.peaklist:
                 peak.setAi((peak.ai - normalization[1]) / normalization[0])
                 peak.setBase((peak.base - normalization[1]) / normalization[0])
-            self.peaklist.refresh()
+            self.peaklist.reset()
         
         # clear buffers
         self.reset()
@@ -2156,7 +2318,8 @@ class scan:
             baselineOffset=baselineOffset, \
             smoothMethod=smoothMethod, \
             smoothWindow=smoothWindow, \
-            smoothCycles=smoothCycles)
+            smoothCycles=smoothCycles
+        )
         
         # update peaklist
         if newPeaklist != None:
@@ -2209,7 +2372,7 @@ class scan:
             baselineData: (list of [x, noiseLevel, noiseWidth]) precalculated baseline
         """
         
-        # label peak
+        # label point
         newPeak = processing.labelPoint(self.points, \
             mz=mz, \
             baselineWindow=baselineWindow, \
@@ -2241,7 +2404,37 @@ class scan:
             mzTolerance=mzTolerance, \
             intTolerance=intTolerance, \
             isotopeShift=isotopeShift \
-            )
+        )
+    # ----
+    
+    
+    def deconvolute(self, massType='mo'):
+        """Recalculate peaklist to singly charged.
+            massType: ('mo' or 'av') mass type used for m/z re-calculation
+        """
+        
+        # delete spectrum points
+        self.points = numpy.array([])
+        
+        # deconvolute peaklist
+        self.peaklist.deconvolute(massType=massType)
+        
+        # clear buffers
+        self.reset()
+    # ----
+    
+    
+    def groupPeaks(self, window, forceWindow=False):
+        """Group peaks within specified window.
+            window: (float) default grouping window
+            forceWindow: (bool) use default window for all peaks instead of fwhm
+        """
+        
+        # group near peaks
+        self.peaklist.groupPeaks(
+            window=window, \
+            forceWindow=forceWindow \
+        )
     # ----
     
     
