@@ -18,6 +18,7 @@
 # load libs
 import re
 import os.path
+import copy
 import numpy
 import math
 from numpy.linalg import solve as solveLinEq
@@ -75,6 +76,8 @@ def mz(mass, charge, currentCharge=0, agentFormula='H', agentCharge=1, massType=
         massType = 0
     elif massType == 'av':
         massType = 1
+    else:
+        raise ValueError, "Mass type must be 'mo' or 'av' --> " + massType
     
     # check agent formula
     if agentFormula != 'e' and not isinstance(agentFormula, objects.compound):
@@ -141,20 +144,18 @@ def coverage(ranges, length, trueIndexes=False):
 # ISOTOPIC PATTERN
 # ----------------
 
-def pattern(compound, fwhm=0.1, threshold=0.01, charge=1, agentFormula='H', agentCharge=1, rounding=7):
+def pattern(compound, fwhm=0.1, threshold=0.01, charge=0, agentFormula='H', agentCharge=1):
     """Calculate isotopic pattern for given compound.
         fwhm: (float) gaussian peak width
-        threshold: (float) intensity threshold
+        threshold: (float) relative intensity threshold in %/100
         charge: (int) charge to be calculated
         agentFormula: (str or formula) charging agent formula
         agentCharge: (int) charging agent charge
-        massType: ('mo' or 'av') used mass type if mass value is float
     """
     
-    finalPattern = []
-    
-    # set internal threshold
-    internalThreshold = threshold/100.
+    # check compound
+    if not isinstance(compound, objects.compound):
+        compound = objects.compound(compound)
     
     # check agent formula
     if agentFormula != 'e' and not isinstance(agentFormula, objects.compound):
@@ -169,13 +170,16 @@ def pattern(compound, fwhm=0.1, threshold=0.01, charge=1, agentFormula='H', agen
     
     # get composition
     composition = compound.composition()
-    
-    # check composition
     for atom in composition:
         if composition[atom] < 0:
             raise ValueError, 'Pattern cannot be calculated for this formula! --> ' + compound.formula()
     
+    # set internal thresholds
+    internalThreshold = threshold/100.
+    groupingWindow = fwhm/4.
+    
     # calculate pattern
+    finalPattern = []
     for atom in composition:
         
         # get isotopic profile for current atom or specified isotope only
@@ -185,74 +189,48 @@ def pattern(compound, fwhm=0.1, threshold=0.01, charge=1, agentFormula='H', agen
         symbol, massNumber, tmp = match.groups()
         if massNumber:
             isotope = blocks.elements[symbol].isotopes[int(massNumber)]
-            atomPattern.append([isotope[0], 1.])
+            atomPattern.append([isotope[0], 1.]) # [mass, abundance]
         else:
             for massNumber, isotope in blocks.elements[atom].isotopes.items():
                 if isotope[1] > 0.:
-                    atomPattern.append(list(isotope))
+                    atomPattern.append(list(isotope)) # [mass, abundance]
         
         # add atoms
         for i in range(atomCount):
             
             CHECK_FORCE_QUIT()
             
-            currentPattern = {}
-            
             # if pattern is empty (first atom) add current atom pattern
             if not finalPattern:
-                finalPattern = atomPattern
-                finalPattern.sort()
+                finalPattern = normalize(atomPattern)
                 continue
             
             # add atom to each peak of final pattern
-            for peak in finalPattern:
+            currentPattern = []
+            for patternIsotope in finalPattern:
                 
                 # skip peak under relevant abundance threshold
-                if peak[1] < internalThreshold:
+                if patternIsotope[1] < internalThreshold:
                     continue
                 
                 # add each isotope of current atom to peak
-                for isotope in atomPattern:
-                    mass = peak[0] + isotope[0]
-                    abundance = peak[1] * isotope[1]
-                    
-                    # add abundance to stored peak or add new peak
-                    mass = round(mass, rounding)
-                    if mass in currentPattern:
-                        currentPattern[mass] += abundance
-                    else:
-                        currentPattern[mass] = abundance
+                for atomIsotope in atomPattern:
+                    mass = patternIsotope[0] + atomIsotope[0]
+                    abundance = patternIsotope[1] * atomIsotope[1]
+                    currentPattern.append([mass, abundance])
             
-            # replace final pattern by current
-            finalPattern = []
-            for mass, abundance in currentPattern.items():
-                finalPattern.append([mass, abundance])
-            finalPattern.sort()
-            
-            # noramlize pattern
-            finalPattern = _normalize(finalPattern)
+            # group isotopes and normalize pattern
+            finalPattern = _groupIsotopes(currentPattern, groupingWindow)
+            finalPattern = normalize(finalPattern)
     
     # correct charge
     if charge:
         for i in range(len(finalPattern)):
             finalPattern[i][0] = (finalPattern[i][0]-ELECTRON_MASS*charge)/abs(charge)
     
-    # collect isotopes according to resolution (FWHM)
-    tol = fwhm/2
-    if charge:
-        tol /= abs(charge)
-    collectedPeaks = [finalPattern[0]]
-    lastPeak = finalPattern[0]
-    for currentPeak in finalPattern[1:]:
-        if (lastPeak[0] + tol) > currentPeak[0]:
-            abundance = lastPeak[1] + currentPeak[1]
-            mass = (lastPeak[0]*lastPeak[1] + currentPeak[0]*currentPeak[1]) / abundance
-            collectedPeaks[-1] = [mass, abundance]
-            lastPeak = [mass, abundance]
-        else:
-            collectedPeaks.append(currentPeak)
-            lastPeak = currentPeak
-    finalPattern = _normalize(collectedPeaks)
+    # group isotopes and normalize pattern
+    finalPattern = _groupIsotopes(finalPattern, groupingWindow)
+    finalPattern = normalize(finalPattern)
     
     # discard peaks below threshold
     filteredPeaks = []
@@ -265,40 +243,64 @@ def pattern(compound, fwhm=0.1, threshold=0.01, charge=1, agentFormula='H', agen
 # ----
 
 
-def profile(peaklist, fwhm=0.1, points=None, noise=None):
+def profile(peaklist, fwhm=0.1, points=10, noise=None, raster=None, forceFwhm=False):
     """Make profile spectrum for given peaklist.
-        peaklist: (mspy.peaklist) peaklist
-        fwhm: (float) peak fwhm
+        peaklist: (list of (mz, intensity) or mspy.peaklist) peaklist
+        fwhm: (float) default peak fwhm
+        points: (int) default number of points per peak width (not used if mzRaster is specified)
         noise: (float) random noise width
-        points: (int) number of points
+        raster: (1D array) m/z values raster
+        forceFwhm: (bool) use default fwhm for all peaks
     """
     
     # check peaklist
     if not isinstance(peaklist, objects.peaklist):
-        raise TypeError, "Peak list must be mspy.peaklist object!"
+        peaklist = objects.peaklist(peaklist)
     
-    # get range and step
-    minX = peaklist[0].mz - 5*fwhm
-    maxX = peaklist[-1].mz + 5*fwhm
-    if points:
-        step = (maxX - minX) / points
-    else:
-        step = fwhm / 10.
+    # get fwhm range
+    minFwhm = None
+    maxFwhm = None
+    if not forceFwhm:
+        for peak in peaklist:
+            if not peak.fwhm:
+                continue
+            if not minFwhm or peak.fwhm < minFwhm:
+                minFwhm = peak.fwhm
+            if not maxFwhm or peak.fwhm > maxFwhm:
+                maxFwhm = peak.fwhm
     
-    # make raster
-    raster = numpy.arange(minX, maxX, step)
+    # use default fwhm range if not set
+    if not minFwhm or not maxFwhm:
+        minFwhm = fwhm
+        maxFwhm = fwhm
+    
+    # get m/z raster
+    if raster == None:
+        mzRange = (peaklist[0].mz - 5*maxFwhm , peaklist[-1].mz + 5*maxFwhm)
+        rasterRange = (minFwhm/points, maxFwhm/points)
+        raster = _makeRaster(mzRange, rasterRange)
+    
+    # get intensity raster
     intensities = numpy.zeros(raster.size, float)
     
-    # calc gaussian peak for each isotope
-    width = fwhm/1.66
+    # calulate gaussian peak for each isotope
     for peak in peaklist:
         
         CHECK_FORCE_QUIT()
         
-        x = _getIndex1D(raster, (peak.mz-5*fwhm))
-        i2 = _getIndex1D(raster, (peak.mz+5*fwhm))
+        # get peak fwhm
+        if peak.fwhm and not forceFwhm:
+            peakFwhm = peak.fwhm
+            peakWidth = peak.fwhm/1.66
+        else:
+            peakFwhm = fwhm
+            peakWidth = fwhm/1.66
+        
+        # calulate peak
+        x = _getIndex1D(raster, (peak.mz-5*peakFwhm))
+        i2 = _getIndex1D(raster, (peak.mz+5*peakFwhm))
         while x < i2:
-            intensities[x] += peak.intensity*numpy.exp(-1*(pow(raster[x]-peak.mz,2))/pow(width,2))
+            intensities[x] += peak.intensity*numpy.exp(-1*(pow(raster[x]-peak.mz,2))/pow(peakWidth,2))
             x += 1
     
     # add random noise
@@ -312,9 +314,12 @@ def profile(peaklist, fwhm=0.1, points=None, noise=None):
     data = data.copy()
     
     # add baseline
-    baselineData = []
-    for peak in peaklist:
-        baselineData.append([peak.mz, -peak.base])
+    baselineData = [[peaklist[0].mz, -peaklist[0].base]]
+    for peak in peaklist[1:]:
+        if baselineData[-1][0] == peak.mz:
+            baselineData[-1][0] -= peak.base
+        else:
+            baselineData.append([peak.mz, -peak.base])
     data = correctBaseline(points=data, baselineData=baselineData)
     
     return data
@@ -381,6 +386,50 @@ def averagine(mass, composition, threshold=0.005, fwhm=0.2):
 # ----
 
 
+def _groupIsotopes(isotopes, window):
+    """Group peaks within specified window.
+        isotopes: (list of [mass, abundance]) isotopes list
+        window: (float) grouping window
+    """
+    
+    isotopes.sort()
+    
+    buff = []
+    buff.append(isotopes[0])
+    
+    for current in isotopes[1:]:
+        previous = buff[-1]
+        if (previous[0] + window) > current[0]:
+            abundance = previous[1] + current[1]
+            mass = (previous[0]*previous[1] + current[0]*current[1]) / abundance
+            buff[-1] = [mass, abundance]
+        else:
+            buff.append(current)
+    
+    return buff
+# ----
+
+
+def _makeRaster(mzRange, rasterRange):
+    """Make m/z raster as linear gradient for given m/z range and edge points differences."""
+    
+    m = (rasterRange[1] - rasterRange[0]) / (mzRange[1] - mzRange[0])
+    b = rasterRange[0] - m * mzRange[0]
+    
+    size = int((mzRange[1] - mzRange[0]) / rasterRange[0]) + 1
+    raster = numpy.zeros(size, float)
+    
+    i = 0
+    x = mzRange[0]
+    while x <= mzRange[1]:
+        raster[i] = x
+        x += m*x + b
+        i += 1
+    
+    return raster[:i].copy()
+# ----
+
+
 
 # SPECTRUM PROCESSING
 # -------------------
@@ -856,53 +905,56 @@ def labelScan(points, pickingHeight=0.75, absThreshold=0., relThreshold=0., snTh
             localMaxima.append(peak)
     
     # make centroides
-    buff = []
-    previous = None
-    for peak in localMaxima:
-        
-        CHECK_FORCE_QUIT()
-        
-        # calc peak height
-        height = ((peak[1]-peak[2]) * pickingHeight) + peak[2]
-        
-        # get peak indexes
-        index = _getIndex2D(points, peak[0])
-        if not (0 < index < len(points)):
-            continue
-        
-        leftIndx = index-1
-        while leftIndx > 0:
-            if points[leftIndx][1] <= height:
-                break
-            leftIndx -= 1
-        
-        rightIndx = index
-        rightMax = len(points) - 1
-        while rightIndx < rightMax:
-            if points[rightIndx][1] <= height:
-                break
-            rightIndx += 1
-        
-        # get mz
-        leftMZ = _interpolateLine(points[leftIndx], points[leftIndx+1], y=height)
-        rightMZ = _interpolateLine(points[rightIndx-1], points[rightIndx], y=height)
-        peak[0] = (leftMZ + rightMZ)/2
-        
-        # get intensity
-        intensity = peakIntensity(points, peak[0])
-        if intensity and intensity <= peak[1]:
-            peak[1] = intensity
-        else:
-            continue
-        
-        # try to group with previous peak
-        if previous != None and leftMZ < previous:
-            if peak[1] > buff[-1][1]:
-                buff[-1] = peak
+    if pickingHeight == 1.:
+        buff = localMaxima
+    else:
+        buff = []
+        previous = None
+        for peak in localMaxima:
+            
+            CHECK_FORCE_QUIT()
+            
+            # calc peak height
+            height = ((peak[1]-peak[2]) * pickingHeight) + peak[2]
+            
+            # get peak indexes
+            index = _getIndex2D(points, peak[0])
+            if not (0 < index < len(points)):
+                continue
+            
+            leftIndx = index-1
+            while leftIndx > 0:
+                if points[leftIndx][1] <= height:
+                    break
+                leftIndx -= 1
+            
+            rightIndx = index
+            rightMax = len(points) - 1
+            while rightIndx < rightMax:
+                if points[rightIndx][1] <= height:
+                    break
+                rightIndx += 1
+            
+            # get mz
+            leftMZ = _interpolateLine(points[leftIndx], points[leftIndx+1], y=height)
+            rightMZ = _interpolateLine(points[rightIndx-1], points[rightIndx], y=height)
+            peak[0] = (leftMZ + rightMZ)/2
+            
+            # get intensity
+            intensity = peakIntensity(points, peak[0])
+            if intensity and intensity <= peak[1]:
+                peak[1] = intensity
+            else:
+                continue
+            
+            # try to group with previous peak
+            if previous != None and leftMZ < previous:
+                if peak[1] > buff[-1][1]:
+                    buff[-1] = peak
+                    previous = rightMZ
+            else:
+                buff.append(peak)
                 previous = rightMZ
-        else:
-            buff.append(peak)
-            previous = rightMZ
     
     CHECK_FORCE_QUIT()
     
@@ -935,10 +987,11 @@ def labelScan(points, pickingHeight=0.75, absThreshold=0., relThreshold=0., snTh
 # ----
 
 
-def envelopeCentroid(isotopes, height=0.5):
+def envelopeCentroid(isotopes, height=0.5, intensity='maximum'):
     """Calculate envelope centroid for given isotopes.
         isotopes: (list of peaks)
         height: (float) intensity to calculate centroid width
+        intensity: (maximum | sum | average) haw to calculate envelope label intensity
     """
     
     # check isotopes
@@ -947,20 +1000,38 @@ def envelopeCentroid(isotopes, height=0.5):
     elif len(isotopes) == 1:
         return isotopes[0]
     
-    # get centroid m/z
+    # get sums
     sumMZ = 0
     sumIntensity = 0
-    basepeak = isotopes[0]
-    
+    sumBase = 0
     for isotope in isotopes:
         sumMZ += isotope.mz * isotope.intensity
         sumIntensity += isotope.intensity
+        sumBase += isotope.base
+    
+    # get m/z
+    mz = sumMZ / sumIntensity
+    
+    # get basepeak
+    basepeak = isotopes[0]
+    for isotope in isotopes:
         if isotope.intensity > basepeak.intensity:
             basepeak = isotope
     
-    mz = sumMZ / sumIntensity
+    # get ai, base and sn
+    base = basepeak.base
+    ai = basepeak.ai
+    sn = basepeak.sn
+    if intensity == 'sum':
+        base = sumBase / len(isotopes)
+        ai = base + sumIntensity
+        sn = (ai - base) * basepeak.sn / (basepeak.ai - basepeak.base)
+    elif intensity == 'average':
+        base = sumBase / len(isotopes)
+        ai = base + sumIntensity / len(isotopes)
+        sn = (ai - base) * basepeak.sn / (basepeak.ai - basepeak.base)
     
-    # get centroid width
+    # get envelope width
     minInt = basepeak.intensity * height
     i1 = None
     i2 = None
@@ -976,13 +1047,14 @@ def envelopeCentroid(isotopes, height=0.5):
         mz1 = _interpolateLine((isotopes[i1-1].mz, isotopes[i1-1].ai), (isotopes[i1].mz, isotopes[i1].ai), y=minInt)
     if i2 < len(isotopes)-1:
         mz2 = _interpolateLine((isotopes[i2].mz, isotopes[i2].ai), (isotopes[i2+1].mz, isotopes[i2+1].ai), y=minInt)
+    
     if mz1 == mz2:
         fwhm = basepeak.fwhm
     else:
         fwhm = mz2 - mz1
     
     # make peak
-    peak = objects.peak(mz=mz, ai=basepeak.ai, base=basepeak.base, sn=basepeak.sn, fwhm=fwhm)
+    peak = objects.peak(mz=mz, ai=ai, base=base, sn=sn, fwhm=fwhm)
     
     return peak
 # ----
@@ -1089,6 +1161,62 @@ def findIsotopes(peaklist, maxCharge=1, mzTolerance=0.15, intTolerance=0.5, isot
 # ----
 
 
+def deconvolute(peaklist, massType='mo'):
+    """Recalculate peaklist to singly charged.
+        peaklist: (mspy.peaklist) peak list to deconvolute
+        massType: ('mo' or 'av') mass type used for m/z re-calculation
+    """
+    
+    # recalculate peaks
+    buff = []
+    for peak in copy.deepcopy(peaklist):
+        
+        CHECK_FORCE_QUIT()
+        
+        # uncharged peak
+        if not peak.charge:
+            continue
+        
+        # correct charge
+        elif abs(peak.charge) == 1:
+            buff.append(peak)
+        
+        # recalculate peak
+        else:
+            
+            # set fwhm
+            if peak.fwhm:
+                newFwhm = abs(peak.fwhm*peak.charge)
+                peak.setFwhm(newFwhm)
+            
+            # set m/z and charge
+            if peak.charge < 0:
+                newMz = mz(mass=peak.mz, charge=-1, currentCharge=peak.charge, massType=massType)
+                peak.setMz(newMz)
+                peak.setCharge(-1)
+            else:
+                newMz = mz(mass=peak.mz, charge=1, currentCharge=peak.charge, massType=massType)
+                peak.setMz(newMz)
+                peak.setCharge(1)
+            
+            # store peak
+            buff.append(peak)
+    
+    # finalize peaks
+    if buff:
+        base = min([p.base for p in buff])
+        for peak in buff:
+            peak.setSN(None)
+            peak.setAi(peak.ai - peak.base + base)
+            peak.setBase(base)
+    
+    # update peaklist
+    peaklist = objects.peaklist(buff)
+    
+    return peaklist
+# ----
+
+
 def correctBaseline(points, window=0.1, smooth=True, offset=0., baselineData=None):
     """Return data points with baseline correction.
         points: (numpy array) spectrum data points
@@ -1107,7 +1235,7 @@ def correctBaseline(points, window=0.1, smooth=True, offset=0., baselineData=Non
     # calculate offsets for points
     offsets = []
     if len(baselineData) == 1:
-        offsets.append((0., baselineData[0][0]))
+        offsets.append((0., baselineData[0][1]))
     else:
         i = 1
         m = (baselineData[i][1] - baselineData[i-1][1])/(baselineData[i][0] - baselineData[i-1][0])
@@ -1197,6 +1325,23 @@ def unifyRaster(pointsA, pointsB):
 # ----
 
 
+def normalize(data):
+    """Normalize data."""
+    
+    # get maximum Y
+    maximum = data[0][1]
+    for item in data:
+        if item[1] > maximum:
+            maximum = item[1]
+    
+    # normalize data data
+    for x in range(len(data)):
+        data[x][1] /= maximum
+    
+    return data
+# ----
+
+
 
 # DATA RE-CALIBRATION
 # -------------------
@@ -1234,7 +1379,7 @@ def _linearModel(params, x):
     
     a, b = params
     return a*x + b
-    
+# ----
 
 
 def _quadraticModel(params, x):
@@ -1242,10 +1387,10 @@ def _quadraticModel(params, x):
     
     a, b, c = params
     return a*x*x + b*x + c
-    
+# ----
 
 
-def _leastSquaresFit(model, parameters, data, max_iterations=None, stopping_limit=0.005):
+def _leastSquaresFit(model, parameters, data, max_iterations=None, stopping_limit=1e-7):
     """General non-linear least-squares fit using the
     Levenberg-Marquardt algorithm and automatic derivatives."""
     
@@ -1275,11 +1420,13 @@ def _leastSquaresFit(model, parameters, data, max_iterations=None, stopping_limi
         if max_iterations is not None and niter == max_iterations:
             pass
     return map(lambda p: p[0], next_p), next_chi_sq[0]
+# ----
 
 
 def _isDerivVar(x):
     """Returns 1 if |x| is a DerivVar object."""
     return hasattr(x,'value') and hasattr(x,'deriv')
+# ----
 
 
 def _chiSquare(model, parameters, data):
@@ -1297,6 +1444,7 @@ def _chiSquare(model, parameters, data):
         d = numpy.array(f[1])/sigma
         alpha = alpha + d[:,numpy.newaxis]*d
     return chi_sq, alpha
+# ----
 
 
 def _mapderiv(func, a, b):
@@ -1306,6 +1454,7 @@ def _mapderiv(func, a, b):
     a = a + (nvars-len(a))*[0]
     b = b + (nvars-len(b))*[0]
     return map(func, a, b)
+# ----
 
 
 class _DerivVar:
@@ -1382,7 +1531,8 @@ class _DerivVar:
     
     def __rpow__(self, other):
         return pow(other, self)
-    
+        
+# ----
 
 
 
@@ -1401,6 +1551,8 @@ def digest(sequence, enzyme, miscleavage=0, allowMods=False, strict=True):
     # check sequence object
     if not isinstance(sequence, objects.sequence):
         raise TypeError, "Cannot digest non-sequence object!"
+    elif not sequence.chain:
+        return []
     
     # get enzyme
     if enzyme in blocks.enzymes:
@@ -1567,12 +1719,13 @@ def fragment(sequence, serie, index=None):
         raise ValueError, 'Unknown fragment terminus! -> ' + serie.terminus
     
     # check fragment specifity
-    for x in range(len(frags)):
-        frags[x].fragmentFiltered = True
-        for aa in serie.specifity:
-            if aa in frags[x].chain:
-                frags[x].fragmentFiltered = False
-                break
+    if serie.specifity:
+        for x in range(len(frags)):
+            frags[x].fragmentFiltered = True
+            for aa in serie.specifity:
+                if aa in frags[x].chain:
+                    frags[x].fragmentFiltered = False
+                    break
     
     # filter nonsense fragments
     if frags and serie.terminus == 'N':
@@ -1599,23 +1752,6 @@ def fragment(sequence, serie, index=None):
 # HELPERS
 # -------
 
-def _normalize(data):
-    """Normalize data."""
-    
-    # get maximum Y
-    maximum = data[0][1]
-    for item in data:
-        if item[1] > maximum:
-            maximum = item[1]
-    
-    # normalize data data
-    for x in range(len(data)):
-        data[x][1] /= maximum
-    
-    return data
-# ----
-
-
 def _interpolateLine(p1, p2, x=None, y=None):
     """Get line interpolated X or Y value."""
     
